# ----------------------------------------------------------------------------------
# Copyright ENS, INRIA, CNRS
# Contributors: Romain Brette (brette@di.ens.fr) and Dan Goodman (goodman@di.ens.fr)
# 
# Brian is a computer program whose purpose is to simulate models
# of biological neural networks.
# 
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# ----------------------------------------------------------------------------------
# 
'''
Neuron StateUpdaters
'''

__all__ = ['StateUpdater', 'LinearStateUpdater', 'NonlinearStateUpdater',
           'SynapticNoise', 'LazyStateUpdater', 'magic_state_updater',
           'FunStateUpdater', 'get_linear_equations']

#from scipy.weave import blitz
from numpy import *
from scipy import linalg
from scipy.linalg import LinAlgError
from scipy import weave
from scipy.optimize import fsolve
import copy
from operator import isSequenceType
from inspection import *
from units import second, mvolt
from clock import guess_clock
import magic
from equations import *
from itertools import count
from units import Quantity
import warnings
from log import *
from globalprefs import *
from experimental.codegen import *
CStateUpdater = PythonStateUpdater = None

def magic_state_updater(model, clock=None, order=1, implicit=False, compile=False, freeze=False, \
                        method=None, check_units=True):
    '''
    Examines the set of differential equations in 'model' (Equations object) and 
    returns a StateUpdater object and the list of dynamic variables.
    For example, the magic_state_updater function can determine if it
    is linear or nonlinear.
    
    Available methods:
    * None: the method is automatically selected
    * linear
    * Euler
    * RK (Runge-Kutta, second order)
    * exponential_Euler
    * nonlinear: automatic selection, but not linear
    '''
    global CStateUpdater, PythonStateUpdater
    if method == 'exponential_Euler':
        implicit = True
        order = 1
    elif method == 'Euler':
        implicit = False
        order = 1
    elif method == 'RK':
        implicit = False
        order = 2
    elif method == 'linear' or method is None:
        pass
    else:
        raise AttributeError, "Unknown integration method!"

    # All the first below should go in Equations
    if not(isinstance(model, Equations)): # a set of equations?
        raise TypeError, "An Equations object must be passed."

    model.prepare(check_units=check_units) # check units and other things
    dynamicvars = model._diffeq_names # Dynamic variables

    # Identify stochastic equations
    noiselist = []
    for statevar in model._diffeq_names:
        f = model._function[statevar]
        x0 = [model._units[var] for var in f.func_code.co_varnames] # init variables
        if depends_on(f, 'xi', x0):
            noiselist.append((statevar, get_global_term(f, 'xi', x0))) # s.d. of noise
            f.func_globals['xi'] = 0 * second ** -.5
        # better: remove in string

    use_codegen = get_global_preference('usecodegen') and get_global_preference('usecodegenstateupdate')
    use_weave = get_global_preference('useweave') and get_global_preference('usecodegenweave')
    if CStateUpdater is None:
        from experimental.codegen.stateupdaters import CStateUpdater, PythonStateUpdater

    # Linearity test
    # insert this in equations
    allow_linear = (method is None) or (method == 'linear')
    if allow_linear and model.is_linear():
        log_info('brian.stateupdater', "Linear model: using exact updates")
        stateupdaterobj = LinearStateUpdater(model, clock=clock)
    else:
        # Nonlinear model - check order of the method
        if implicit: # implicit integration schemes
            if model.is_conditionally_linear():
                log_info('brian.stateupdater', "Using exponential Euler")
                if not use_codegen:
                    stateupdaterobj = ExponentialEulerStateUpdater(model, clock=clock, compile=compile, freeze=freeze)
                elif use_weave:
                    stateupdaterobj = CStateUpdater(model, exp_euler_scheme, clock=clock, freeze=freeze)
                    log_warn('brian.stateupdater', 'Using codegen CStateUpdater')
                else:
                    stateupdaterobj = PythonStateUpdater(model, exp_euler_scheme, clock=clock, freeze=freeze)
                    log_warn('brian.stateupdater', 'Using codegen PythonStateUpdater')
            else:
                raise TypeError, "General implicit methods are not implemented yet."
        else: # explicit method
            if order == 1:
                if not use_codegen:
                    stateupdaterobj = NonlinearStateUpdater(model, clock=clock, compile=compile, freeze=freeze)
                elif use_weave:
                    stateupdaterobj = CStateUpdater(model, euler_scheme, clock=clock, freeze=freeze)
                    log_warn('brian.stateupdater', 'Using codegen CStateUpdater')
                else:
                    stateupdaterobj = PythonStateUpdater(model, euler_scheme, clock=clock, freeze=freeze)
                    log_warn('brian.stateupdater', 'Using codegen PythonStateUpdater')
            elif order == 2:
                if not use_codegen:
                    stateupdaterobj = RK2StateUpdater(model, clock=clock, compile=compile, freeze=freeze)
                elif use_weave:
                    stateupdaterobj = CStateUpdater(model, rk2_scheme, clock=clock, freeze=freeze)
                    log_warn('brian.stateupdater', 'Using codegen CStateUpdater')
                else:
                    stateupdaterobj = PythonStateUpdater(model, rk2_scheme, clock=clock, freeze=freeze)
                    log_warn('brian.stateupdater', 'Using codegen PythonStateUpdater')
            else:
                raise TypeError, "Methods with order greater than 2 are not implemented yet."

    # Insert noise
    for var, sigma in noiselist:
        # TODO: noise with mu = 0
        i = dynamicvars.index(var)
        stateupdaterobj = SynapticNoise(stateupdaterobj, i, 0 * model._units[var] / second, sigma, clock)

    return stateupdaterobj, dynamicvars

# TODO: StateUpdater should be lazy by default
class StateUpdater(object):
    '''
    A callable state update mechanism.
    By default, a leaky integrate-and-fire model with zero resting potential
    and unit time constant.
    Warning: to update the state matrix, use the slice operation, e.g.
    S[:]=0 (not S=0)
    otherwise operations are not done in place (a new object is created),
    so that all views are compromised (the reference to the data changes).
    '''
    def __init__(self, clock=None):
        '''
        Default model: dv/dt=-v
        '''
        if clock == None:
            self.update_factor = exp(-clock.dt) # The update matrix
        else:
            raise TypeError, "A time reference must be passed."

    def rest(self, P):
        '''
        Sets the variables at rest.
        P is the neuron group.
        '''
        warnings.warn('Rest is not implemented for this model')

    def __call__(self, P):
        '''
        Updates the state variables.
        Careful here: always use the slice operation for affectations.
        P is the neuron group.
        '''
        P._S[:] *= self.update_factor

    def __repr__(self):
        return 'Leaky integrate-and-fire StateUpdater'

    def __len__(self):
        '''
        Number of state variables
        '''
        return 1

#def get_linear_equations(eqs):
#    '''
#    Returns the matrices M and B for the linear model dX/dt = M(X-B),
#    where eqs is an Equations object. 
#    '''
#    # Otherwise assumes it is given in functional form
#    n=len(eqs._diffeq_names) # number of state variables
#    dynamicvars=eqs._diffeq_names
#    # Calculate B
#    AB=zeros((n,1))
#    d=dict.fromkeys(dynamicvars)
#    for j in range(n):
#        d[dynamicvars[j]]=0.*eqs._units[dynamicvars[j]]
#    for var,i in zip(dynamicvars,count()):
#        AB[i]=-eqs.apply(var,d)
#    # Calculate A
#    M=zeros((n,n))
#    for i in range(n):
#        for j in range(n):
#            d[dynamicvars[j]]=0.*eqs._units[dynamicvars[j]]
#        if isinstance(eqs._units[dynamicvars[i]],Quantity):
#            d[dynamicvars[i]]=Quantity.with_dimensions(1.,eqs._units[dynamicvars[i]].get_dimensions())
#        else:
#            d[dynamicvars[i]]=1.
#        for var,j in zip(dynamicvars,count()):
#            M[j,i]=eqs.apply(var,d)+AB[j]
#    M-=eye(n)*1e-10 # quick dirty fix for problem of constant derivatives; dimension = Hz
#    B=linalg.lstsq(M,AB)[0] # We use this instead of solve in case M is degenerate
#    return M,B

def get_linear_equations(eqs):
    '''
    Returns the matrices M and B for the linear model dX/dt = M(X-B),
    where eqs is an Equations object. 
    '''
    # Otherwise assumes it is given in functional form
    n = len(eqs._diffeq_names) # number of state variables
    dynamicvars = eqs._diffeq_names
    # Calculate B
    AB = zeros((n, 1))
    d = dict.fromkeys(dynamicvars)
    for j in range(n):
        d[dynamicvars[j]] = 0. * eqs._units[dynamicvars[j]]
    for var, i in zip(dynamicvars, count()):
        AB[i] = -eqs.apply(var, d)
    # Calculate A
    M = zeros((n, n))
    for i in range(n):
        for j in range(n):
            d[dynamicvars[j]] = 0. * eqs._units[dynamicvars[j]]
        if isinstance(eqs._units[dynamicvars[i]], Quantity):
            d[dynamicvars[i]] = Quantity.with_dimensions(1., eqs._units[dynamicvars[i]].get_dimensions())
        else:
            d[dynamicvars[i]] = 1.
        for var, j in zip(dynamicvars, count()):
            M[j, i] = eqs.apply(var, d) + AB[j]
    #M-=eye(n)*1e-10 # quick dirty fix for problem of constant derivatives; dimension = Hz
    #B=linalg.lstsq(M,AB)[0] # We use this instead of solve in case M is degenerate
    B = linalg.solve(M, AB) # We use this instead of solve in case M is degenerate
    return M, B

def get_linear_equations_solution_numerically(eqs, dt):
    # Otherwise assumes it is given in functional form
    n = len(eqs._diffeq_names) # number of state variables
    dynamicvars = eqs._diffeq_names
    # Calculate B
    AB = zeros((n, 1))
    d = dict.fromkeys(dynamicvars)
    for j in range(n):
        d[dynamicvars[j]] = 0. * eqs._units[dynamicvars[j]]
    for var, i in zip(dynamicvars, count()):
        AB[i] = -eqs.apply(var, d)
    # Calculate A
    M = zeros((n, n))
    for i in range(n):
        for j in range(n):
            d[dynamicvars[j]] = 0. * eqs._units[dynamicvars[j]]
        if isinstance(eqs._units[dynamicvars[i]], Quantity):
            d[dynamicvars[i]] = Quantity.with_dimensions(1., eqs._units[dynamicvars[i]].get_dimensions())
        else:
            d[dynamicvars[i]] = 1.
        for var, j in zip(dynamicvars, count()):
            M[j, i] = eqs.apply(var, d) + AB[j]
    #B=linalg.solve(M,AB)
    numeulersteps = 100
    deltat = dt / numeulersteps
    E = eye(n) + deltat * M
    C = eye(n)
    D = zeros((n, 1))
    for step in xrange(numeulersteps):
        C, D = dot(E, C), dot(E, D) - AB * deltat
    return C, D
    #return M,B

set_global_preferences(useweave_linear_diffeq=False)
define_global_preference('useweave_linear_diffeq', 'False',
                           desc="""
                                  Whether to use weave C++ acceleration for the solution
                                  of linear differential equations. Note that on some
                                  platforms, typically older ones, this is faster and on
                                  some platforms, typically new ones, this is actually
                                  slower.
                                  """)


class LinearStateUpdater(StateUpdater):
    '''
    A linear model with dynamics dX/dt = M(X-B) or dX/dt = MX.
    
    **Initialised as:** ::
    
        LinearStateUpdater(M[,B[,clock]])
    
    with arguments:
    
    ``M``
        Matrix defining the differential equation.
    ``B``
        Optional linear term in the differential equation.
    ``clock``
        Optional clock.
    
    Computes an update matrix A=exp(M dt) for the linear system,
    and performs the update step.
    
    TODO: more mathematical details? 
    '''
    #TODO: sparse linear models (e.g. cable equations)
    def __init__(self, M, B=None, clock=None):
        '''
        Initialize a linear model with dynamics dX/dt = M(X-B) or dX/dt = MX,
        where B is a column vector.
        TODO: more checks
        TODO: rest
        '''
        self._useaccel = get_global_preference('useweave_linear_diffeq')
        self._cpp_compiler = get_global_preference('weavecompiler')
        self._extra_compile_args = ['-O3']
        if self._cpp_compiler == 'gcc':
            self._extra_compile_args += get_global_preference('gcc_options') # ['-march=native', '-ffast-math']
        self._useB = False
        if clock == None:
            clock = guess_clock()
        if isinstance(M, ndarray):
            self.A = linalg.expm(M * clock.dt)
            self.B = B
        elif isinstance(M, Equations):
            try:
                M, self.B = get_linear_equations(M)
                self.A = linalg.expm(M * clock.dt)
                #self.A=array(self.A,single)
                if self.B is not None:
                    self._C = -dot(self.A, self.B) + self.B
                    #self._C=array(self._C,single)
                    self._useB = True
                else:
                    self._useB = False
            except LinAlgError:
                log_info('brian.stateupdater', 'Solving linear equations numerically')
                self.A, self._C = get_linear_equations_solution_numerically(M, clock.dt)
                self.B = NotImplemented # raises error on trying to use this
                self._useB = True
        # note the numpy dot command works faster if self.A has C ordering compared
        # to fortran ordering (although maybe this depends on which implementation
        # of BLAS you're using). The difference is only significant in small
        # calculations because making a copy of self.A is usually not serious, its
        # size is only the number of variables, not the number of neurons.
        self.A = array(self.A, order='C')
        if self._useB:
            self._C = array(self._C, order='C')

    def rest(self, P):
        if self._useB:
            if self.B is NotImplemented:
                raise NotImplementedError, \
                    "The resting potential cannot be found because the equations are degenerate " + \
                    "(most likely because they include a parameter)"
            P._S[:] = self.B
        else:
            P._S[:] = 0

    def __call__(self, P):
        '''
        Updates the state variables.
        Careful here: always use the slice operation for affectations.
        P is the neuron group.
        '''
        if self._useB: # This could be removed
            if not self._useaccel:
                #P._S[:]=dot(self.A,P._S)+self._C
                P._S[:] = dot(self.A, P._S)
                #P._S = dot(self.A,P._S)
                #P._S += self._C
                add(P._S, self._C, P._S)
                #P._S[:]=dot(self.A,P._S-self.B)+self.B
            else:
                m = len(self)
                S = P._S
                n = S.shape[1] #n = len(P)
                A = self.A
                c = self._C
                code = '''
                double x[m]; 
                for(int i=0;i<n;i++)  
                {
                    for(int j=0;j<m;j++)
                    {
                        x[j] = c(j);
                        for(int k=0;k<m;k++)
                           x[j] += A(j,k) * S(k,i);
                    }
                    for(int j=0;j<m;j++)
                        S(j,i) = x[j];
                } 
                '''
                weave.inline(code, ['n', 'm', 'S', 'A', 'c'],
                             compiler=self._cpp_compiler,
                             type_converters=weave.converters.blitz,
                             extra_compile_args=self._extra_compile_args)
        else:
            if not self._useaccel:
                P._S[:] = dot(self.A, P._S)
            else:
                n = len(P)
                m = len(self)
                S = P._S
                A = self.A
                code = '''
                double x[m]; 
                for(int i=0;i<n;i++)  
                {
                    for(int j=0;j<m;j++)
                    {
                        x[j] = 0.0;
                        for(int k=0;k<m;k++)
                           x[j] += A(j,k) * S(k,i);
                    }
                    for(int j=0;j<m;j++)
                        S(j,i) = x[j];
                } 
                '''
                weave.inline(code, ['n', 'm', 'S', 'A'],
                             compiler=self._cpp_compiler,
                             type_converters=weave.converters.blitz,
                             extra_compile_args=self._extra_compile_args)

    def __repr__(self):
        return 'Linear StateUpdater with ' + str(len(self)) + ' state variables'

    def __len__(self):
        '''
        Number of state variables
        '''
        return self.A.shape[0]


class NonlinearStateUpdater(StateUpdater):
    '''
    A nonlinear model with dynamics dX/dt = f(X).
    Uses an Equations object.
    By default, uses Euler integration.
    '''
    def __init__(self, eqs, clock=None, compile=False, freeze=False):
        '''
        Initialize a nonlinear model with dynamics dX/dt = f(X).
        f is given as an Equations object (see examples).
        If compile is True, a Python code object is compiled.
        '''
        # TODO: global pref?
        self.eqs = eqs
        self.optimized = compile
        self._first_time = True
        if freeze:
            self.eqs.compile_functions(freeze=freeze)
        self._frozen=freeze
        if compile:
            self._code = self.eqs.forward_euler_code()

    def rest(self, P):
        '''
        Sets the variables at rest.
        '''
        for name, value in self.eqs.fixed_point().iteritems():
            P.state(name)[:] = value

    def __call__(self, P):
        '''
        Updates the state variables.
        Careful here: always use the slice operation for affectations.
        P is the neuron group.
        Euler integration.
        '''
        #if self.optimized==False:
        #    self.eqs.optimize(len(P))
        #    self.optimized=True
        # TODO: do these operations once
        #states=dict.fromkeys(self.eqs.dynamicvars)
        # store that in the neurongroup?
        if self.optimized:
            if self._first_time:
                self._first_time = False
                P._dS = 0 * P._S
            dt = P.clock._dt
            t = P.clock.t
            exec(self._code)
        else:
            states = dict.fromkeys(self.eqs._diffeq_names) # ={}?
            #for var in self.eqs.dynamicvars:
            for var in self.eqs._diffeq_names:
                states[var] = P.state_(var)
            if self._frozen:
                states['t']=P.clock._t # without units
            else:
                states['t'] = P.clock.t #time
            self.eqs.forward_euler(states, P.clock._dt)

    def __repr__(self):
        return 'Nonlinear StateUpdater with ' + str(len(self)) + ' state variables'

    def __len__(self):
        '''
        Number of state variables
        '''
        return len(self.eqs)


class RK2StateUpdater(NonlinearStateUpdater):
    '''
    A nonlinear model with dynamics dX/dt = f(X).
    Uses an Equations object.
    Uses Runge-Kutta midpoint integration (second order).
    '''
    def __init__(self, eqs, clock=None, compile=False, freeze=False):
        '''
        Initialize a nonlinear model with dynamics dX/dt = f(X).
        f is given as an Equations object (see examples).
        If compile is True, a Python code object is compiled.
        '''
        # TODO: global pref?
        self.eqs = eqs
        self._first_time = True
        if freeze:
            self.eqs.compile_functions(freeze=freeze)
        self._frozen=freeze
        if compile:
            warnings.warn('Compilation is not implemented yet for RK2 integration.')

    def __call__(self, P):
        '''
        Updates the state variables.
        Careful here: always use the slice operation for affectations.
        P is the neuron group.
        Euler integration.
        '''
        states = dict.fromkeys(self.eqs._diffeq_names) # ={}?
        #for var in self.eqs.dynamicvars:
        for var in self.eqs._diffeq_names:
            states[var] = P.state_(var)
        if self._frozen:
            states['t']=P.clock._t # without units
        else:
            states['t'] = P.clock.t #time
        self.eqs.Runge_Kutta2(states, P.clock._dt)


class ExponentialEulerStateUpdater(NonlinearStateUpdater):
    def __init__(self, eqs, clock=None, compile=False, freeze=False):
        '''
        Initialize a nonlinear model with dynamics dX/dt = f(X).
        '''
        # TODO: global pref?
        self.eqs = eqs
        self.optimized = compile
        self._first_time = True
        if freeze:
            self.eqs.compile_functions(freeze=freeze)
        self._frozen=freeze
        if compile:
            self._code = self.eqs.exponential_euler_code()

    def __call__(self, P):
        '''
        Updates the state variables.
        Careful here: always use the slice operation for affectations.
        P is the neuron group.
        '''
        if self.optimized:
            if self._first_time:
                self._first_time = False
                P._dS = P._S.copy()
            dt = P.clock._dt
            t = P.clock.t
            exec(self._code)
        else:
            # TODO: do these operations once
            states = dict.fromkeys(self.eqs._diffeq_names) #={}?
            for var in self.eqs._diffeq_names:
                states[var] = P.state_(var)
            if self._frozen:
                states['t']=P.clock._t # without units
            else:
                states['t'] = P.clock.t #time
            self.eqs.exponential_euler(states, P.clock._dt)


class SynapticNoise(StateUpdater):
    '''
    Synaptic noise mechanism, plugged into another StateUpdater.
    '''
    def __init__(self, baseupdater, nstate, mu, sigma, clock=None):
        '''
        baseupdater = source neuron StateUpdater
        nstate = index of synaptic state variable
        mu = mean synaptic input rate (per ms)
        sigma = s.d. of synaptic input per ms^{1/2}
        '''
        self.baseupdater = baseupdater
        self.nstate = nstate
        if clock == None:
            clock = guess_clock()
        if clock:
            # TODO: check units
            self.mu = mu * clock.dt
            self.sigma = sigma * clock.dt ** .5
        else:
            raise TypeError, "A time reference must be passed."

    def rest(self, P):
        self.baseupdater.rest(P)

    def __call__(self, P):
        '''
        Updates the state variables.
        Careful here: always use the slice operation for affectations.
        P is the neuron group.
        '''
        self.baseupdater(P) # update the underlying model
        P._S[self.nstate, :] += self.mu + random.randn(P._S.shape[1]) * self.sigma

    def __repr__(self):
        return self.baseupdater.__repr__() + ' with synaptic noise on variable ' + str(self.nstate)

    def __len__(self):
        '''
        Number of state variables
        '''
        return len(self.baseupdater)


class LazyStateUpdater(StateUpdater):
    '''
    A StateUpdater that does nothing.
    
    **Initialised as:** ::
    
        LazyStateUpdater([numstatevariables=1[,clock]])
        
    with arguments:
    
    ``numstatevariables``
        The number of state variables to create.
    ``clock``
        An optional clock to determine when it updates,
        although the update function does nothing so...
    '''
#    Alternatively, we might replace the parent StateUpdater class by this and
#    write a basic leaky integrator class.
    def __init__(self, numstatevariables=1, clock=None):
        self._N = numstatevariables
        pass

    def __call__(self, P):
        '''
        Updates the state variables.
        '''
        pass

    def __repr__(self):
        return 'Lazy StateUpdater (does nothing)'

    def __len__(self):
        '''
        Number of state variables
        '''
        return self._N

# UNTESTED
class FunStateUpdater(StateUpdater):
    """
    A StateUpdater that calls a function at each update step
    
    A StateUpdater function takes one argument, the neuron group
    that is being updated.
    """
    def __init__(self, func, numstates, clock=None):
        self.clock = guess_clock(clock)
        self.func = func
        self.numstates = numstates

    def __call__(self, P):
        self.func(P)

    def __repr__(self):
        return "Function StateUpdater, function " + self.func.__name__

    def __len__(self):
        return self.numstates



from brian import *
from brian.experimental.codegen.threshold import *

eqs = Equations('''
V : volt
Vt : volt
''')

threshold = '''
    (V>Vt)&(V>Vt)
'''

print '**** C CODE *****'
print generate_c_threshold(eqs, threshold)
#
print '**** PYTHON CODE ****'
print generate_python_threshold(eqs, threshold)

GP = NeuronGroup(10, eqs, threshold=PythonThreshold(threshold), reset=0 * volt)
GC = NeuronGroup(10, eqs, threshold=CThreshold(threshold), reset=0 * volt)
MP = SpikeMonitor(GP)
MC = SpikeMonitor(GC)

GP.V[[1, 2, 4, 8]] = 2
GP.Vt = 1 * volt
GP.Vt[[2, 8]] = 3
GC.V[[1, 2, 4, 8]] = 2
GC.Vt = 1 * volt
GC.Vt[[2, 8]] = 3

run(1 * ms)

print GP.V
print GP.Vt
print MP.spikes
print
print GC.V
print GC.Vt
print MC.spikes



from brian import *
from multiprocessing.connection import Listener, Client
import select

server = ('localhost', 2719)
server_authkey = 'brian'

eqs = '''
dv/dt = (ge+gi-(v+49*mV))/(20*ms) : volt
dge/dt = -ge/(5*ms) : volt
dgi/dt = -gi/(10*ms) : volt
'''
P = NeuronGroup(4000, model=eqs,
              threshold= -50 * mV, reset= -60 * mV)
P.v = -60 * mV + 10 * mV * rand(len(P))
Pe = P.subgroup(3200)
Pi = P.subgroup(800)
Ce = Connection(Pe, P, 'ge', weight=1.62 * mV, sparseness=0.02)
Ci = Connection(Pi, P, 'gi', weight= -9 * mV, sparseness=0.02)
M = SpikeMonitor(P)
@network_operation(clock=EventClock(dt=1 * second))
def clearspikes():
    M.reinit()
    print 'Simulated', defaultclock.t

listener = Listener(server, authkey=server_authkey)
#conn = listener.accept()
conn = None

global_ns = globals()
local_ns = locals()

@network_operation
def server_check():
    global conn
    if conn is None:
        socket = listener._listener._socket
        sel, _, _ = select.select([socket], [], [], 0)
        if len(sel):
            conn = listener.accept()
    if conn is None:
        return
    if not conn.poll():
        return
    job = conn.recv()
    jobtype, jobargs = job
    if jobtype == 'exec':
        exec jobargs in global_ns, local_ns
        result = None
    elif jobtype == 'eval':
        result = eval(jobargs, global_ns, local_ns)
    conn.send(result)

run(1e10 * second)

print 'Finished!'



'''
Example of the use of the class :class:`~brian.hears.Gammatone` available in the
library. It implements a fitlerbank of IIR gammatone filters as 
described  in Slaney, M., 1993, "An Efficient Implementation of the
Patterson-Holdsworth Auditory Filter Bank". Apple Computer Technical Report #35. 
In this example, a white noise is filtered by a gammatone filterbank and the
resulting cochleogram is plotted.
'''
from brian import *
from brian.hears import *
from matplotlib import pyplot

sound = whitenoise(100*ms).ramp()
sound.level = 50*dB

nbr_center_frequencies = 50
b1 = 1.019  #factor determining the time constant of the filters
#center frequencies with a spacing following an ERB scale
center_frequencies = erbspace(100*Hz, 1000*Hz, nbr_center_frequencies)
gammatone = Gammatone(sound, center_frequencies, b=b1)

gt_mon = gammatone.process()

figure()
imshow(gt_mon.T, aspect='auto', origin='lower left',
       extent=(0, sound.duration/ms,
               center_frequencies[0], center_frequencies[-1]))
pyplot.yscale('log')
title('Cochleogram')
ylabel('Frequency (Hz)')
xlabel('Time (ms)')

show()



"""
Offline compensation example
Input current: 1 second of filtered noise
Raw trace: Rothman Mannis model of the cochlear nucleus
"""
from brian import *
from brian.experimental.modelfitting import *
import numpy

if __name__ == '__main__':

    # load input current
    input = numpy.load("compensation_I.npy")
    
    # load neuron trace
    neurontrace = numpy.load("compensation_Vm.npy")
    
    # raw trace: add electrode resistance to neuron trace
    rawtrace = neurontrace + 500 * Mohm * input
    
    # time
    t = linspace(0.0, 1.0, 10000)
    
    # compensate the raw trace using 100 particles and 20 iterations
    comp, full, electrode, results = compensate(input,
                                                rawtrace,
                                                gpu=1,
                                                popsize=100,
                                                maxiter=20)
    
    print "The procedure found an electrode resistance Re=%.1f MOhm" \
        % (results["Re"]/Mohm)
    
    # plot the input current
    subplot(211)
    plot(t, input, 'k')
    xlabel("Time (s)")
    ylabel("Input current (A)")
    
    # plot the neuron trace, the raw trace, and the compensated trace
    subplot(212)
    plot(t, rawtrace, 'k', label="raw")
    plot(t, neurontrace, 'g', label="neuron")
    plot(t, comp, 'r', label="compensated")
    xlabel("Time (s)")
    ylabel("Membrane potential (V)")
    legend()
    
    show()



from brian import *

set_global_preferences(useweave=True)

inp = PoissonGroup(1, rates=30 * Hz)

###### Standard ODE approach
eqs = '''
    dV/dt = -V/(0.5*second) : volt
    '''
G = NeuronGroup(1, model=eqs, threshold=10 * mV, reset=0 * mV)
C = Connection(inp, G)
C.connect_full(inp, G, weight=2 * mV)
M = StateMonitor(G, 'V', record=True)

###### Event driven approach, maybe faster if spikes are rare
eqs_ed = '''
    V : volt
    exc : volt
    '''
G_ed = NeuronGroup(1, model=eqs_ed, threshold=10 * mV, reset=0 * mV)
C_ed = Connection(inp, G_ed, 'exc')
C_ed.connect_full(inp, G_ed, weight=2 * mV)
M_ed = StateMonitor(G_ed, 'V', record=True)
G_ed.last_event_time = 0.0
@network_operation(when='before_groups')
def update_G_ed():
    if G_ed.exc_[0] > 1e-10:
        t = defaultclock._t
        G_ed.V_[0] = G_ed.V_[0] * exp((G_ed.last_event_time - t) / 0.5) + G_ed.exc_[0]
        G_ed.exc_[0] = 0.0
        G_ed.last_event_time = t

####### Run and plot
run(1 * second)
plot(M.times, M[0])
plot(M_ed.times, M_ed[0])
show()



'''
Very short example program.
'''

from numpy import *
#from pylab import *
import time
import random

second = 1.
ms = 0.001
mV = 0.001

N = 4000
dt = 0.1 * ms
T = 100 * ms
numsteps = int(T / dt)
Vr = -60 * mV
Vt = -50 * mV

S = zeros((3, N))
A = array([[ 0.99501248, 0.00493794, 0.00496265],
 [ 0.     , 0.98019867, 0.        ],
 [ 0.  , 0.  , 0.99004983]])
_C = array([[ -2.44388520e-04],
 [ -8.58745657e-21],
 [  6.90431479e-20]])

S[0, :] = [random.uniform(Vr, Vt) for _ in xrange(N)]

l = [[] for _ in range(N)]

t = 0
start = time.time()
Nspike = 0
for _ in xrange(numsteps):
    S[:] = dot(A, S) + _C
    I = where(S[0, :] > Vt)[0]
    Nspike += len(I)
    S[0, I] = Vr
    t += dt
#    for i in range(N):
#        l[i].append(S[0,i])
print time.time() - start
print Nspike

#for i in range(N):
#    plot(l[i])
#show()

#from brian import *
#
#eqs='''
#dv/dt = (ge+gi-(v+49*mV))/(20*ms) : volt
#dge/dt = -ge/(5*ms) : volt
#dgi/dt = -gi/(10*ms) : volt
#'''
#
#P=NeuronGroup(4000,model=eqs,
#              threshold=-50*mV,reset=-60*mV)

#P.v=-60*mV+10*mV*rand(len(P))
#Pe=P.subgroup(3200)
#Pi=P.subgroup(800)
#
#Ce=Connection(Pe,P,'ge')
#Ci=Connection(Pi,P,'gi')
#Ce.connect_random(Pe, P, 0.02,weight=1.62*mV)
#Ci.connect_random(Pi, P, 0.02,weight=-9*mV)
#
#M=SpikeMonitor(P)
#
#run(1*second)
#raster_plot(M)
#show()



'''
This class defines data structures for the Synapses object.

** ConstructionSparseStateVector

The main one is the ConstructionSparseStateVector, look into it's doc
for details!

TODO:
- Finish to write the SparseStateVector (not dynamic), and use it
accordingly in the Synapses code
- More thorough test of set/get


- think about the issue discussed with Romain: hashtable for lookup of values? keep sorted spikes?


** ParameterVector (worst name in the world)

It is only returned by the getattr method of the Synapse object and is
used for the fancy slicing (see doc)

TODO:
- Check the behavior for 3-D slicing

'''
import numpy as np

INITIAL_LEN = 2 # the initial length of the ConstructionSparseStateVector


########################## UTILS ##########################

def dtype2str(dtype):
    '''
    Converts a numpy.dtype object into a string for indexing in
    SparseStateVector's value dictionnary
    '''
    return str(dtype)[13:-2]
    
def ensure_iterable(x):
    '''
    Makes sure the value x is iterable, if it's not (a float/int,
    etc...) turn it into an array
    '''
    if not np.iterable(x):
        return np.array([x])
    else:
        return x

def slice2range(sl, n):
    '''
    Given a slice object and a length, returns the array of indices
    concerned by the slice.
    Useful in all the getitem methods!
    '''
    tmp = np.arange(n, dtype = np.int32)
    return ensure_iterable(tmp[sl])

########################## STATE VECTORS ##########################

class SparseStateVector(object):
    '''
    CURRENTLY NOT USED
    
    Compressed (i.e not dynamic) state vectors array
    
    Basically should act as a labeled, static ndarray with possibly different dtypes for different rows.
    '''
    def __init__(self, data, dtypes, row_indices, nvalues, labels = []):
        
        self.dtypes = dtypes
        self.nrows = len(dtypes)
        self.nvalues = nvalues
        
        if labels:
            if not len(labels) == self.nrows:
                raise ValueError('Wrong number of labels')
        
        self.labels = labels
        
        self.row_indices = row_indices
        self._values_dict = data
        
    def __getitem__(self, key):
        ## Before going further I have to know exactly how i'll be using those
        if not isinstance(key, tuple) and len(key) == 2:
            raise IndexError('Wrong index')
        else:
            rows = slice2range(key[0], self.shape[0])
            cols = slice2range(key[1], self.shape[1])
            
            new_dtypes = [typ for i,typ in enumerate(self.dtypes) if i in rows]
            
            for typ in cur_dtypes:
                pass
            
    @property
    def shape(self):
        return (self.nrows, self.nvalues)
            
            
            
class ConstructionSparseStateVector(object):
    '''
    This class is used to construct the state vector structure
    underlying the synapses object. It is basically a
    2-D array with possibly different types per row. The rows in this
    object are also labeled, and can be retrieved and set via v.fieldname.
    More importantly, it is dynamic, and doubles its length every time a
    value is entered that reaches the maximum size.
    

    
    ** Initialization ** 
    
    V = ConstructionSparseStateVector(nrows, dtype = int, labels = None)
    
    ``nrows'' is the number of rows (fields) of the data structure
    
    ``dtype'' is either a single, or a tuple of numpy dtypes that
    correspond to the wanted dtypes for each row
    
    ``labels'' when set, one can get each row by accessing the
    object's attribute of the same name
    
    Example:
    ConstructionSparseStateVector(3, dtype = (int32, float32, int32),
                                     labels = ['pre', 'weight', 'post'])
                                     
    
    ** Dynamic Structure **
    
    One can add new values (columns) on the fly to the data structure
    by append them, for example is V is the structure in the above
    example, one can add 2 values as follows:
    
    values = [np.ones(2, dtype = int32), 
              np.randn(2, dtype = float32), 
              np.zeros(2, dtype = int32)]
              
    V.append(values)

    This will add two 1s in the 'pre' field, two random numbers in the
    'weight' field etc...

    ** Accessing the fields **
    
    The recommended way of doing so is to label the field and use its
    label as an attribute. In the above example, we'd use 
    V.pre
    To get the ``pre'' field.
    
    ** To Static ** 
    
    ..automethod:: compress
    
    TODO: Use that!
    
    ** Internals! **
    
    ``_values_dict''
    
    Internally the arrays of the different dtypes are stored into a
    dictionnary, and values of the same type in the
    same array.
    
    The function dtype2str converts the dtype object into a string
    (int32 -> 'int32') and the int32 values of the structure are
    accessed internally by self._values_dict['int32']
    
    ``nvalues''
    
    Because of the dynamic structure, the arrays in the dictionnary
    can be bigger than there is data in them, hence we keep the number
    of relevant values.
    
    ``row_indices''
    
    row_indices[i] keeps track of the row index of the row i in it's
    dedicated array.
    to get row i we thus do
    
    tmp = self._values_dict[dtype2str(self.dtype[i])]
    return tmp[row_indices[i],:]
    
    I think that's about it!
    '''
    def __init__(self, nrows, dtype = int, labels = None):
        
        self.nrows = nrows
        
        # check dtype:
        if not isinstance(dtype, tuple):
            dtype = (dtype,)*nrows
        if not len(dtype) == nrows:
            raise ValueError('Wrong number of dtypes')
        self.dtypes = dtype
        
        # possibly row labels
        if labels:
            if not len(labels) == nrows:
                raise ValueError('Wrong number of labels')
        self.labels = labels
     
        # take same dtypes and gather them
        nrows_per_typ, self.unique_typ = [], []
        self.row_indices = np.zeros(len(self.dtypes), dtype = int) # will hold the number of the row in the dedicated dtype array
        for i,typ in enumerate(self.dtypes):
            if not typ in self.unique_typ:
                self.unique_typ.append(typ)
                nrows_per_typ.append(0)
            idtyp = self.unique_typ.index(typ)
            self.row_indices[i] = nrows_per_typ[idtyp]
            nrows_per_typ[idtyp] += 1
            

        self._values_dict = {}
        # now construct the state nd arrays
        for i,typ in enumerate(self.unique_typ):
            self._values_dict[dtype2str(typ)] = np.zeros((nrows_per_typ[i], INITIAL_LEN), 
                                                          dtype = typ)
            
            
        self.nvalues = 0 # initially no values instantiated
        self.datastruct_size = INITIAL_LEN # datastructure size is the size (nrows) of the arrays

    def append(self, values):
        '''
        Values must be a list of 1-D arrays of the right type containing the values to be inserted.
        '''
        # check the values shapes
        if not len(values) == self.nrows:
            raise ValueError('Wrong number of values')

        # where to insert the values
        initial_index = self.nvalues
        final_index = initial_index + len(values[0])

        # if the new size is bigger that the datastruct size, resize the arrays
        if final_index > self.datastruct_size:

            newsize = 2**np.ceil(np.log2(final_index))

            for typ in self.unique_typ:

                if not dtype2str(typ)[:5] == 'float':
                    mul = -1
                else:
                    mul = np.inf
                    
                newarray = mul * np.ones((self._values_dict[dtype2str(typ)].shape[0], 
                                             newsize), dtype = typ)
                newarray[:, :self.nvalues] = self._values_dict[dtype2str(typ)][:, :self.nvalues]
                
                self._values_dict[dtype2str(typ)] = newarray
                #np.resize(self._values_dict[dtype2str(typ)], (self._values_dict[dtype2str(typ)].shape[0], newsize))
            self.datastruct_size = newsize
        
        # insert the values
        for i, typ in enumerate(self.dtypes):
            self._values_dict[dtype2str(typ)][self.row_indices[i], initial_index:final_index] = values[i]

        self.nvalues = final_index
            
    def compress(self):
        compressed_values_dict = {}
        for i, typ in enumerate(self.dtypes):
            compressed_values_dict[dtype2str(typ)] = self._values_dict[dtype2str(typ)][:, :self.nvalues]
        
        return SparseStateVector(compressed_values_dict, self.dtypes, self.row_indices, self.nvalues, labels = self.labels)
        
     
    def __getattr__(self, name, default = ''):
        if name == '_allstates':
            return self._values_dict['float32']
        if name in self.labels:
            i = self.labels.index(name)
            # where to look
            cur_typ = self.dtypes[i]
            row = self.row_indices[i]
            return self._values_dict[dtype2str(cur_typ)][row, :self.nvalues]
        
        try:
            self.__dict__[name]
        except KeyError:
            raise AttributeError('State vector object doesn\'t have a '+name+' attribute')
        
    def __getitem__(self, name):
        if name in self.labels:
            i = self.labels.index(name)
            # where to look
            cur_typ = self.dtypes[i]
            row = self.row_indices[i]
            return self._values_dict[dtype2str(cur_typ)][row, :self.nvalues]
        else:
            raise KeyError('Item '+name+' not in statevector')
            
        
class ParameterVector(object):
    '''
    Helpful for the setattr of Synapses, 
    
    indeed we want 
    
    synapses.w[slice_pre, slice_post] = 1
    
    to mean
    
    for all the presyn neurons of slice_pre for which there is a synapse, resp, postsyn, set the weight to 1.
    
    hence synapses.w must be of a special class whose __setitem__ method is overriden, otherwise the returned value of synapses.w isn't the right shape etc...
    
    Also we want:

    synapses.w[slice_pre, slice_post, :5] = 2

    to mean

    the 5 first synapses between slice_pre and slice_post neurons = 2
    
    KWD ARGS
    
    ``delay_dt'' is here because in the case of delays, then the value (float with unit) must be converted into timesteps (int), hence to do this this object has to know about the dt.

    '''
    def __init__(self, data, groups_shape, existing_synapses, delay_dt = None):
        self.data = data
        self.groups_shape = groups_shape
        self.existing_synapses = existing_synapses
        self.delay_dt = delay_dt
    
    def __setitem__(self, key, value):
        if isinstance(key, tuple):
            indices = []
            for pre in slice2range(key[0], self.groups_shape[0]):
                for post in slice2range(key[1], self.groups_shape[1]):
                    index = self.existing_synapses == np.array([[pre],[post]])
                    index = np.nonzero(np.prod(index, axis = 0))[0]
                    if len(key) > 2:
                        index = index[slice2range(key[2], len(index))]
                    for id in index:
                        indices.append(id)
            if self.delay_dt:
                value = np.array(np.round(value/self.delay_dt), dtype = self.data.dtype)
            self.data[indices] = value
        else:
            raise IndexError
        

    def __getitem__(self, key):
        return self.data.__getitem__(key)

    def __repr__(self):
        return self.data.__repr__()

    def __str__(self):
        return self.data.__repr__()



from brian import *
from numpy import *
import numpy
import array as pyarray
import time
import struct
try:
    import pygame
    have_pygame = True
except ImportError:
    have_pygame = False
try:
    from scikits.samplerate import resample
    have_scikits_samplerate = True
except (ImportError, ValueError):
    have_scikits_samplerate = False
from bufferable import Bufferable
from prefs import get_samplerate
from db import dB, dB_type, dB_error, gain
from scipy.signal import fftconvolve, lfilter
from scipy.misc import factorial

__all__ = ['BaseSound', 'Sound',
           'pinknoise','brownnoise','powerlawnoise',
           'whitenoise', 'irns', 'irno', 
           'tone', 'click', 'clicks', 'silence', 'sequence', 'harmoniccomplex',
           'loadsound', 'savesound', 'play', 'vowel'
           ]


_mixer_status = [-1,-1]

class BaseSound(Bufferable):
    '''
    Base class for Sound and OnlineSound
    '''
    pass

class Sound(BaseSound, numpy.ndarray):
    '''
    Class for working with sounds, including loading/saving, manipulating and playing.

    For an overview, see :ref:`sounds_overview`.
    
    **Initialisation**
    
    The following arguments are used to initialise a sound object
    
    ``data``
        Can be a filename, an array, a function or a sequence (list or tuple).
        If its a filename, the sound file (WAV or AIFF) will be loaded. If its
        an array, it should have shape ``(nsamples, nchannels)``. If its a
        function, it should be a function f(t). If its a sequence, the items
        in the sequence can be filenames, functions, arrays or Sound objects.
        The output will be a multi-channel sound with channels the corresponding
        sound for each element of the sequence. 
    ``samplerate=None``
        The samplerate, if necessary, will use the default (for an array or
        function) or the samplerate of the data (for a filename).
    ``duration=None``
        The duration of the sound, if initialising with a function.
  
    **Loading, saving and playing**
    
    .. automethod:: load
    .. automethod:: save
    .. automethod:: play

    **Properties**
    
    .. autoattribute:: duration
    .. autoattribute:: nsamples
    .. autoattribute:: nchannels
    .. autoattribute:: times
    .. autoattribute:: left
    .. autoattribute:: right
    .. automethod:: channel
    
    **Generating sounds**
    
    All sound generating methods can be used with durations arguments in samples (int) or units (e.g. 500*ms). One can also set the number of channels by setting the keyword argument nchannels to the desired value. Notice that for noise the channels will be generated independantly.
    
    .. automethod:: tone
    .. automethod:: whitenoise
    .. automethod:: powerlawnoise
    .. automethod:: brownnoise
    .. automethod:: pinknoise
    .. automethod:: silence
    .. automethod:: click
    .. automethod:: clicks
    .. automethod:: harmoniccomplex
    .. automethod:: vowel
    
    **Timing and sequencing**
    
    .. automethod:: sequence(*sounds, samplerate=None)
    .. automethod:: repeat
    .. automethod:: extended
    .. automethod:: shifted
    .. automethod:: resized

    **Slicing**
    

    One can slice sound objects in various ways, for example ``sound[100*ms:200*ms]``
    returns the part of the sound between 100 ms and 200 ms (not including the
    right hand end point). If the sound is less than 200 ms long it will be
    zero padded. You can also set values using slicing, e.g.
    ``sound[:50*ms] = 0`` will silence the first 50 ms of the sound. The syntax
    is the same as usual for Python slicing. In addition, you can select a
    subset of the channels by doing, for example, ``sound[:, -5:]`` would be
    the last 5 channels. For time indices, either times or samples can be given,
    e.g. ``sound[:100]`` gives the first 100 samples. In addition, steps can
    be used for example to reverse a sound as ``sound[::-1]``.
    
    **Arithmetic operations**
    
    Standard arithemetical operations and numpy functions work as you would
    expect with sounds, e.g. ``sound1+sound2``, ``3*sound`` or ``abs(sound)``.
    
    **Level**
    
    .. autoattribute:: level
    .. automethod:: atlevel
    .. autoattribute:: maxlevel
    .. automethod:: atmaxlevel
    
    **Ramping**
    
    .. automethod:: ramp
    .. automethod:: ramped
    
    **Plotting**
    
    .. automethod:: spectrogram
    .. automethod:: spectrum
    '''
    duration = property(fget=lambda self:len(self) / self.samplerate,
                        doc='The length of the sound in seconds.')
    nsamples = property(fget=lambda self:len(self),
                        doc='The number of samples in the sound.')
    times = property(fget=lambda self:arange(len(self), dtype=float) / self.samplerate,
                     doc='An array of times (in seconds) corresponding to each sample.')
    nchannels = property(fget=lambda self:self.shape[1],
                         doc='The number of channels in the sound.')
    left = property(fget=lambda self:self.channel(0),
                    doc='The left channel for a stereo sound.')
    right = property(fget=lambda self:self.channel(1),
                     doc='The right channel for a stereo sound.')

    @check_units(samplerate=Hz, duration=second)
    def __new__(cls, data, samplerate=None, duration=None):
        if isinstance(data, numpy.ndarray):
            samplerate = get_samplerate(samplerate)
#            if samplerate is None:
#                raise ValueError('Must specify samplerate to initialise Sound with array.')
            if duration is not None:
                raise ValueError('Cannot specify duration when initialising Sound with array.')
            x = array(data, dtype=float)
        elif isinstance(data, str):
            if duration is not None:
                raise ValueError('Cannot specify duration when initialising Sound from file.')
            if samplerate is not None:
                raise ValueError('Cannot specify samplerate when initialising Sound from a file.')
            x = Sound.load(data)
            samplerate = x.samplerate
        elif callable(data):
            samplerate = get_samplerate(samplerate)
#            if samplerate is None:
#                raise ValueError('Must specify samplerate to initialise Sound with function.')
            if duration is None:
                raise ValueError('Must specify duration to initialise Sound with function.')
            L = int(rint(duration * samplerate))
            t = arange(L, dtype=float) / samplerate
            x = data(t)
        elif isinstance(data, (list, tuple)):
            kwds = {}
            if samplerate is not None:
                kwds['samplerate'] = samplerate
            if duration is not None:
                kwds['duration'] = duration
            channels = tuple(Sound(c, **kwds) for c in data)
            x = hstack(channels)
            samplerate = channels[0].samplerate
        else:
            raise TypeError('Cannot initialise Sound with data of class ' + str(data.__class__))
        if len(x.shape)==1:
            x.shape = (len(x), 1)
        x = x.view(cls)
        x.samplerate = samplerate
        x.buffer_init()
        return x

    def __array_wrap__(self, obj, context=None):
        handled = False
        x = numpy.ndarray.__array_wrap__(self, obj, context)
        if not hasattr(x, 'samplerate') and hasattr(self, 'samplerate'):
            x.samplerate = self.samplerate
        if context is not None:
            ufunc = context[0]
            args = context[1]
        return x
    
    def __array_finalize__(self,obj):
        if obj is None: return
        self.samplerate = getattr(obj, 'samplerate', None)
        
    def buffer_init(self):
        pass
        
    def buffer_fetch(self, start, end):
        if start<0:
            raise IndexError('Can only use positive indices in buffer.')
        samples = end-start
        X = asarray(self)[start:end, :]
        if X.shape[0]<samples:
            X = vstack((X, zeros((samples-X.shape[0], X.shape[1]))))
        return X

    def channel(self, n):
        '''
        Returns the nth channel of the sound.
        '''
        return Sound(self[:, n], self.samplerate)

    def __add__(self, other):
        if isinstance(other, Sound):
            if int(other.samplerate) > int(self.samplerate):
                self = self.resample(other.samplerate)
            elif int(other.samplerate) < int(self.samplerate):
                other = other.resample(self.samplerate)

            if len(self) > len(other):
                other = other.resized(len(self))
            elif len(self) < len(other):
                self = self.resized(len(other))

            return Sound(numpy.ndarray.__add__(self, other), samplerate=self.samplerate)
        else:
            x = numpy.ndarray.__add__(self, other)
            return Sound(x, self.samplerate)
    __radd__ = __add__
    
    # getslice and setslice need to be implemented for compatibility reasons,
    # but __getitem__ covers all the functionality so we just use that
    def __getslice__(self, start, stop):
        return self.__getitem__(slice(start, stop))

    def __setslice__(self, start, stop, seq):
        return self.__setitem__(slice(start, stop), seq)
    
    def __getitem__(self,key):
        channel = slice(None)
        if isinstance(key, tuple):
            channel = key[1]
            key = key[0]

        if isinstance(key, int):
            return np.ndarray.__getitem__(self, key)
        if isinstance(key, float):
            return np.ndarray.__getitem__(self, round(key*self.samplerate))

        sliceattr = [v for v in [key.start, key.stop] if v is not None]
        slicedims = array([units.have_same_dimensions(flag, second) for flag in sliceattr])

        attrisint = array([isinstance(v, int) for v in sliceattr])
        s = sum(attrisint)
        if s!=0 and s!=len(sliceattr):
            raise ValueError('Slice attributes must be all ints or all times')
        if s==len(sliceattr): # all ints
            start = key.start or 0
            stop = key.stop or self.shape[0]
            step = key.step or 1
            if start>=0 and stop<=self.shape[0]:
                return Sound(np.ndarray.__getitem__(self, (key, channel)),
                             self.samplerate)
            else:
                startpad = max(-start, 0)
                endpad = max(stop-self.shape[0], 0)
                startmid = max(start, 0)
                endmid = min(stop, self.shape[0])
                atstart = zeros((startpad, self.shape[1]))
                atend = zeros((endpad, self.shape[1]))
                return Sound(vstack((atstart,
                                     asarray(self)[startmid:endmid:step],
                                     atend)), self.samplerate)
        if not slicedims.all():
            raise DimensionMismatchError('Slicing',
                                        *[units.get_unit(d) for d in sliceattr])
        
        start = key.start or 0*msecond
        stop = key.stop or self.duration
        step = key.step or 1
        if int(step)!=step:
            #resampling
            raise NotImplementedError
        start = int(rint(start*self.samplerate))
        stop = int(rint(stop*self.samplerate))
        return self.__getitem__((slice(start,stop,step),channel))
    
    def __setitem__(self,key,value):
        channel=slice(None)
        if isinstance(key,tuple):
            channel=key[1]
            key=key[0]
        
        if isinstance(key,int) or isinstance(key,float):
            return np.ndarray.__setitem__(self,(key,channel),value)
        if isinstance(key,Quantity):
            return np.ndarray.__setitem__(self,(int(rint(key*self.samplerate)),channel),value)

        sliceattr = [v for v in [key.start, key.step, key.stop] if v is not None]
        slicedims=array([units.have_same_dimensions(flag,second) for flag in sliceattr])
        if not slicedims.any():
            # If value is a mono sound its shape will be (N, 1) but the numpy
            # setitem will have shape (N,) so in this case it's a shape mismatch
            # so we squeeze the array to make sure this doesn't happen.
            if isinstance(value,Sound) and channel!=slice(None):
                value=value.squeeze()
            return asarray(self).__setitem__((key,channel),value)

#            print np.ndarray.__getitem__(self, (key, channel)).shape
#            print key, channel
#            return np.ndarray.__setitem__(self, (key, channel), value)
        if not slicedims.all():
            raise DimensionMismatchError('Slicing',*[units.get_unit(d) for d in sliceattr])

        if key.__getattribute__('step') is not None:
            # resampling?
            raise NotImplementedError
        
        start = key.start
        stop = key.stop or self.duration
        if (start is not None and start<0*ms) or stop > self.duration:
            raise IndexError('Slice bigger than Sound object')
        if start is not None: start = int(rint(start*self.samplerate))
        #if (stop is not None) and (not stop=self.duration): stop = int(stop*self.samplerate)
        stop = int(rint(stop*self.samplerate))
        return self.__setitem__((slice(start,stop),channel),value)

    def extended(self, duration):
        '''
        Returns the Sound with length extended by the given duration, which
        can be the number of samples or a length of time in seconds.
        '''
        duration = get_duration(duration, self.samplerate)
        return self[:self.nsamples+duration]

    def resized(self, L):
        '''
        Returns the Sound with length extended (or contracted) to have L samples.
        '''
        if L == len(self):
            return self
        elif L < len(self):
            return Sound(self[:L, :], samplerate=self.samplerate)
        else:
            padding = zeros((L - len(self), self.nchannels))
            return Sound(concatenate((self, padding)), samplerate=self.samplerate)

    def shifted(self, duration, fractional=False, filter_length=2048):
        '''
        Returns the sound delayed by duration, which can be the number of
        samples or a length of time in seconds. Normally, only integer
        numbers of samples will be used, but if ``fractional=True`` then
        the filtering method from
        `http://www.labbookpages.co.uk/audio/beamforming/fractionalDelay.html <http://www.labbookpages.co.uk/audio/beamforming/fractionalDelay.html>`__
        will be used (introducing some small numerical errors). With this
        method, you can specify the ``filter_length``, larger values are
        slower but more accurate, especially at higher frequencies. The large
        default value of 2048 samples provides good accuracy for sounds with
        frequencies above 20 Hz, but not for lower frequency sounds. If you are
        restricted to high frequency sounds, a smaller value will be more
        efficient. Note that if ``fractional=True`` then
        ``duration`` is assumed to be a time not a number of samples.
        '''
        if not fractional:
            if not isinstance(duration, int):
                duration = int(rint(duration*self.samplerate))
            if duration>=0:
                y = vstack((zeros((duration, self.nchannels)), self))
                return Sound(y, samplerate=self.samplerate)
            else:
                return self[-duration:, :]
        else:
            if self.nchannels>1:
                sounds = [self.channel(i).shifted(duration, fractional=True, filter_length=filter_length) for i in xrange(self.nchannels)]
                return Sound(array(sounds), samplerate=self.samplerate)
            # Adapted from
            # http://www.labbookpages.co.uk/audio/beamforming/fractionalDelay.html
            delay = duration*self.samplerate
            if delay>=0:
                idelay = int(delay)
            elif delay<0:
                idelay = -int(-delay)
            delay -= idelay
            centre_tap = filter_length // 2
            t = arange(filter_length)
            x = t-delay
            if abs(round(delay)-delay)<1e-10:
                tap_weight = array(x==centre_tap, dtype=float)
            else:
                sinc = sin(pi*(x-centre_tap))/(pi*(x-centre_tap))
                window = 0.54-0.46*cos(2.0*pi*(x+0.5)/filter_length) # Hamming window
                tap_weight = window*sinc
            if filter_length<256:
                y = convolve(tap_weight, self.flatten())
            else:
                y = fftconvolve(tap_weight, self.flatten())
            y = y[filter_length/2:-filter_length/2]
            sound = Sound(y, self.samplerate)
            sound = sound.shifted(idelay)
            return sound

    def repeat(self, n):
        '''
        Repeats the sound n times
        '''
        x = vstack((self,)*n)
        return Sound(x, samplerate=self.samplerate)

    ### TODO: test this - I haven't installed scikits.samplerate on windows
    # it should work, according to the documentation 2D arrays are acceptable
    # in the format we use fof sounds here
    @check_units(samplerate=Hz)
    def resample(self, samplerate, resample_type='sinc_best'):
        '''
        Returns a resampled version of the sound.
        '''
        if not have_scikits_samplerate:
            raise ImportError('Need scikits.samplerate package for resampling')
        y = array(resample(self, float(samplerate / self.samplerate), resample_type),
                  dtype=float64)
        return Sound(y, samplerate=samplerate)

    def _init_mixer(self):
        global _mixer_status
        if _mixer_status==[-1,-1] or _mixer_status[0]!=self.nchannels or _mixer_status != self.samplerate:
            pygame.mixer.quit()
            pygame.mixer.init(int(self.samplerate), -16, self.nchannels)
            _mixer_status=[self.nchannels,self.samplerate]
    
    def play(self, normalise=False, sleep=False):
        '''
        Plays the sound (normalised to avoid clipping if required). If
        sleep=True then the function will wait until the sound has finished
        playing before returning.
        '''
        if self.nchannels>2:
            raise ValueError("Can only play sounds with 1 or 2 channels.")
        self._init_mixer()
        if normalise:
            a = amax(abs(self))
        else:
            a = 1
        x = array((2 ** 15 - 1) * clip(self / a, -1, 1), dtype=int16)
        if self.nchannels==1:
            x.shape = x.size
        # Make sure pygame receives an array in C-order        
        x = pygame.sndarray.make_sound(ascontiguousarray(x))
        x.play()
        if sleep:
            time.sleep(self.duration)

    def spectrogram(self, low=None, high=None, log_power=True, other = None,  **kwds):
        '''
        Plots a spectrogram of the sound
        
        Arguments:
        
        ``low=None``, ``high=None``
            If these are left unspecified, it shows the full spectrogram,
            otherwise it shows only between ``low`` and ``high`` in Hz.
        ``log_power=True``
            If True the colour represents the log of the power.
        ``**kwds``
            Are passed to Pylab's ``specgram`` command.
        
        Returns the values returned by pylab's ``specgram``, namely
        ``(pxx, freqs, bins, im)`` where ``pxx`` is a 2D array of powers,
        ``freqs`` is the corresponding frequencies, ``bins`` are the time bins,
        and ``im`` is the image axis.
        '''
        if self.nchannels>1:
            raise ValueError('Can only plot spectrograms for mono sounds.')
        if other is not None:
            x = self.flatten()-other.flatten()
        else:
            x = self.flatten()
        pxx, freqs, bins, im = specgram(x, Fs=self.samplerate, **kwds)
        if low is not None or high is not None:
            restricted = True
            if low is None:
                low = 0*Hz
            if high is None:
                high = amax(freqs)*Hz
            I = logical_and(low <= freqs, freqs <= high)
            I2 = where(I)[0]
            I2 = [max(min(I2) - 1, 0), min(max(I2) + 1, len(freqs) - 1)]
            Z = pxx[I2[0]:I2[-1], :]
        else:
            restricted = False
            Z = pxx
        if log_power:
            Z[Z < 1e-20] = 1e-20 # no zeros because we take logs
            Z = 10 * log10(Z)
        Z = flipud(Z)
        if restricted:
            imshow(Z, extent=(0, amax(bins), freqs[I2[0]], freqs[I2[-1]]), aspect='auto')
        else:
            imshow(Z, extent=(0, amax(bins), freqs[0], freqs[-1]), aspect='auto')
        xlabel('Time (s)')
        ylabel('Frequency (Hz)')
        return (pxx, freqs, bins, im)

    def spectrum(self, low=None, high=None, log_power=True, display=False):
        '''
        Returns the spectrum of the sound and optionally plots it.
        
        Arguments:
        
        ``low``, ``high`` 
            If these are left unspecified, it shows the full spectrum,
            otherwise it shows only between ``low`` and ``high`` in Hz.
        ``log_power=True``
            If True it returns the log of the power.
        ``display=False``
            Whether to plot the output.
        
        Returns ``(Z, freqs, phase)``
        where ``Z`` is a 1D array of powers, ``freqs`` is the corresponding
        frequencies, ``phase`` is the unwrapped phase of spectrum.
        '''
        if self.nchannels>1:
            raise ValueError('Can only plot spectrum for mono sounds.')
        sp = numpy.fft.fft(array(self))
        freqs = array(range(len(sp)), dtype=float64) / len(sp) * float64(self.samplerate)
        pxx = abs(sp) ** 2
        phase = unwrap(mod(angle(sp), 2 * pi))
        if low is not None or high is not None:
            restricted = True
            if low is None:
                low = 0*Hz
            if high is None:
                high = amax(freqs)*Hz
            I = logical_and(low <= freqs, freqs <= high)
            I2 = where(I)[0]
            Z = pxx[I2]
            freqs = freqs[I2]
            phase = phase[I2]
        else:
            restricted = False
            Z = pxx
        if log_power:
            Z[Z < 1e-20] = 1e-20 # no zeros because we take logs
            Z = 10 * log10(Z)
        if display:
            subplot(211)
            semilogx(freqs, Z)
            ticks_freqs = 32000 * 2 ** -array(range(18), dtype=float64)
            xticks(ticks_freqs, map(str, ticks_freqs))
            grid()
            xlim((freqs[0], freqs[-1]))
            xlabel('Frequency (Hz)')
            ylabel('Power (dB/Hz)') if log_power else ylabel('Power')
            subplot(212)
            semilogx(freqs, phase)
            ticks_freqs = 32000 * 2 ** -array(range(18), dtype=float64)
            xticks(ticks_freqs, map(str, ticks_freqs))
            grid()
            xlim((freqs[0], freqs[-1]))
            xlabel('Frequency (Hz)')
            ylabel('Phase (rad)')
            show()
        return (Z, freqs, phase)

    def get_level(self):
        '''
        Returns level in dB SPL (RMS) assuming array is in Pascals.
        In the case of multi-channel sounds, returns an array of levels
        for each channel, otherwise returns a float.
        '''
        if self.nchannels==1:
            rms_value = sqrt(mean((asarray(self)-mean(asarray(self)))**2))
            rms_dB = 20.0*log10(rms_value/2e-5)
            return rms_dB*dB
        else:
            return array(tuple(self.channel(i).get_level() for i in xrange(self.nchannels)))

    def set_level(self, level):
        '''
        Sets level in dB SPL (RMS) assuming array is in Pascals. ``level``
        should be a value in dB, or a tuple of levels, one for each channel.
        '''
        rms_dB = self.get_level()
        if self.nchannels>1:
            level = array(level)
            if level.size==1:
                level = level.repeat(self.nchannels)
            level = reshape(level, (1, self.nchannels))
            rms_dB = reshape(rms_dB, (1, self.nchannels))
        else:
            if not isinstance(level, dB_type):
                raise dB_error('Must specify level in dB')
            rms_dB = float(rms_dB)
            level = float(level)
        gain = 10**((level-rms_dB)/20.)
        self *= gain

    level = property(fget=get_level, fset=set_level, doc='''
        Can be used to get or set the level of a sound, which should be in dB.
        For single channel sounds a value in dB is used, for multiple channel
        sounds a value in dB can be used for setting the level (all channels
        will be set to the same level), or a list/tuple/array of levels. It
        is assumed that the unit of the sound is Pascals.
        ''')
    
    def atlevel(self, level):
        '''
        Returns the sound at the given level in dB SPL (RMS) assuming array is
        in Pascals. ``level`` should be a value in dB, or a tuple of levels,
        one for each channel.
        '''
        newsound = self.copy()
        newsound.level = level
        return newsound
    
    def get_maxlevel(self):
        return amax(self.level)*dB
    
    def set_maxlevel(self, level):
        self.level += level-self.maxlevel
        
    maxlevel = property(fget=get_maxlevel, fset=set_maxlevel, doc='''
        Can be used to set or get the maximum level of a sound. For mono
        sounds, this is the same as the level, but for multichannel sounds
        it is the maximum level across the channels. Relative level differences
        will be preserved. The specified level should be a value in dB, and it
        is assumed that the unit of the sound is Pascals. 
        ''')

    def atmaxlevel(self, level):
        '''
        Returns the sound with the maximum level across channels set to the
        given level. Relative level differences will be preserved. The specified
        level should be a value in dB and it is assumed that the unit of the
        sound is Pascals.
        '''
        newsound = self.copy()
        newsound.maxlevel = level
        return newsound
            
    def ramp(self, when='onset', duration=10*ms, envelope=None, inplace=True):
        '''
        Adds a ramp on/off to the sound
        
        ``when='onset'``
            Can take values 'onset', 'offset' or 'both'
        ``duration=10*ms``
            The time over which the ramping happens
        ``envelope``
            A ramping function, if not specified uses ``sin(pi*t/2)**2``. The
            function should be a function of one variable ``t`` ranging from
            0 to 1, and should increase from ``f(0)=0`` to ``f(0)=1``. The
            reverse is applied for the offset ramp.
        ``inplace``
            Whether to apply ramping to current sound or return a new array.
        '''
        when = when.lower().strip()
        if envelope is None: envelope = lambda t:sin(pi * t / 2) ** 2
        if not isinstance(duration, int):
            sz = int(rint(duration * self.samplerate))
        else:
            sz = duration
        multiplier = envelope(reshape(linspace(0.0, 1.0, sz), (sz, 1)))
        if inplace:
            target = self
        else:
            target = Sound(copy(self), self.samplerate)
        if when == 'onset' or when == 'both':
            target[:sz, :] *= multiplier
        if when == 'offset' or when == 'both':
            target[target.nsamples-sz:, :] *= multiplier[::-1]
        return target
    
    def ramped(self, when='onset', duration=10*ms, envelope=None):
        '''
        Returns a ramped version of the sound (see :meth:`Sound.ramp`).
        '''
        return self.ramp(when=when, duration=duration, envelope=envelope, inplace=False)

    def fft(self,n=None):
        '''
        Performs an n-point FFT on the sound object, that is an array of the same size containing the DFT of each channel. n defaults to the number of samples of the sound, but can be changed manually setting the ``n`` keyword argument
        '''
        if n is None:
            n=self.shape[0]
        res=zeros(n,self.nchannels)
        for i in range(self.nchannels):
            res[:,i]=fft(asarray(self)[:,i].flatten(),n=n)
        return res
            
        
    
    @staticmethod
    def tone(frequency, duration, phase=0, samplerate=None, nchannels=1):
        '''
        Returns a pure tone at frequency for duration, using the default
        samplerate or the given one. The ``frequency`` and ``phase`` parameters
        can be single values, in which case multiple channels can be
        specified with the ``nchannels`` argument, or they can be sequences
        (lists/tuples/arrays) in which case there is one frequency or phase for
        each channel.
        '''
        samplerate = get_samplerate(samplerate)
        duration = get_duration(duration,samplerate)
        frequency = array(frequency)
        phase = array(phase)
        if frequency.size>nchannels and nchannels==1:
            nchannels = frequency.size
        if phase.size>nchannels and nchannels==1:
            nchannels = phase.size
        if frequency.size==nchannels:
            frequency.shape = (nchannels, 1)
        if phase.size==nchannels:
            phase.shape =(nchannels, 1)
        t = arange(0, duration, 1)/samplerate
        t.shape = (t.size, 1) # ensures C-order (in contrast to tile(...).T )
        x = sin(phase + 2.0 * pi * frequency * tile(t, (1, nchannels)))
        return Sound(x, samplerate)

    @staticmethod
    def harmoniccomplex(f0, duration, amplitude=1, phase=0, samplerate=None, nchannels=1):
        '''
        Returns a harmonic complex composed of pure tones at integer multiples
        of the fundamental frequency ``f0``. 
        The ``amplitude`` and
        ``phase`` keywords can be set to either a single value or an
        array of values. In the former case the value is set for all
        harmonics, and harmonics up the the sampling frequency are
        generated. In the latter each harmonic parameter is set
        separately, and the number of harmonics generated corresponds
        to the length of the array.
        '''
        samplerate=get_samplerate(samplerate)

        phases = np.array(phase).flatten()
        amplitudes = np.array(amplitude).flatten()
        
        if len(phases)>1 or len(amplitudes)>1:
            if (len(phases)>1 and len(amplitudes)>1) and (len(phases) != len(amplitudes)):
                raise ValueError('Please specify the same number of phases and amplitudes')        
            Nharmonics = max(len(phases),len(amplitudes)) 
        else:
            Nharmonics = int(np.floor( samplerate/(2*f0) ) )
            
        if len(phases) == 1:
            phases = np.tile(phase, Nharmonics)
        if len(amplitudes) == 1:
            amplitudes = np.tile(amplitude, Nharmonics)
            
        x = amplitudes[0]*tone(f0, duration, phase = phases[0], 
                               samplerate = samplerate, nchannels = nchannels)
        for i in range(1,Nharmonics):
            x += amplitudes[i]*tone((i+1)*f0, duration, phase = phases[i], 
                                    samplerate = samplerate, nchannels = nchannels)
        return Sound(x,samplerate)
    
    @staticmethod
    def whitenoise(duration, samplerate=None, nchannels=1):
        '''
        Returns a white noise. If the samplerate is not specified, the global
        default value will be used.
        '''
        samplerate = get_samplerate(samplerate)
        duration = get_duration(duration,samplerate)
        x = randn(duration,nchannels)
        return Sound(x, samplerate)

    @staticmethod
    def powerlawnoise(duration, alpha, samplerate=None, nchannels=1,normalise=False):
        '''
        Returns a power-law noise for the given duration. Spectral density per unit of bandwidth scales as 1/(f**alpha).
        
        Sample usage::
        
            noise = powerlawnoise(200*ms, 1, samplerate=44100*Hz)
        
        Arguments:
        
        ``duration`` 
            Duration of the desired output.
        ``alpha``
            Power law exponent.
        ``samplerate``
            Desired output samplerate
        '''
        samplerate = get_samplerate(samplerate)
        duration = get_duration(duration,samplerate)
        
        # Adapted from http://www.eng.ox.ac.uk/samp/software/powernoise/powernoise.m
        # Little MA et al. (2007), "Exploiting nonlinear recurrence and fractal
        # scaling properties for voice disorder detection", Biomed Eng Online, 6:23
        n=duration
        n2=floor(n/2)
        
        f=array(fftfreq(n,d=1.0/samplerate), dtype=complex)
        f.shape=(len(f),1)
        f=tile(f,(1,nchannels))
        
        if n%2==1:
            z=(randn(n2,nchannels)+1j*randn(n2,nchannels))
            a2=1.0/( f[1:(n2+1),:]**(alpha/2.0))
        else:
            z=(randn(n2-1,nchannels)+1j*randn(n2-1,nchannels))
            a2=1.0/(f[1:n2,:]**(alpha/2.0))
        
        a2*=z
        
        if n%2==1:
            d=vstack((ones((1,nchannels)),a2,
                      flipud(conj(a2))))
        else:
            d=vstack((ones((1,nchannels)),a2,
                      1.0/( abs(f[n2])**(alpha/2.0) )*
                      randn(1,nchannels),
                      flipud(conj(a2))))
        
        
        x=real(ifft(d.flatten()))                  
        x.shape=(n,nchannels)

        if normalise:
            for i in range(nchannels):
                #x[:,i]=normalise_rms(x[:,i])
                x[:,i] = ((x[:,i] - amin(x[:,i]))/(amax(x[:,i]) - amin(x[:,i])) - 0.5) * 2;
        
        return Sound(x,samplerate)
    
            
    @staticmethod
    def pinknoise(duration, samplerate=None, nchannels=1, normalise=False):
        '''
        Returns pink noise, i.e :func:`powerlawnoise` with alpha=1
        '''
        return Sound.powerlawnoise(duration,1.0,samplerate=samplerate,normalise=False)
    
    @staticmethod
    def brownnoise(duration, samplerate=None, nchannels=1,normalise=False):
        '''
        Returns brown noise, i.e :func:`powerlawnoise` with alpha=2
        '''
        return Sound.powerlawnoise(duration,2.0,samplerate=samplerate,normalise=False)
    
    @staticmethod
    def irns(delay, gain, niter, duration, samplerate=None, nchannels=1):
        '''
        Returns an IRN_S noise. The iterated ripple noise is obtained trough
        a cascade of gain and delay filtering. 
        For more details: see Yost 1996 or chapter 15 in Hartman Sound Signal Sensation.
        '''
        samplerate = get_samplerate(samplerate)
        noise=Sound.whitenoise(duration)
        splrate=noise.samplerate
        x=array(noise.T)[0]
        IRNfft=np.fft.fft(x)
        Nspl,spl_dur=len(IRNfft),float(1.0/splrate)
        w=2*pi*fftfreq(Nspl,spl_dur)
        d=float(delay)
        for k in range(1,niter+1):
            nchoosek=factorial(niter)/(factorial(niter-k)*factorial(k))
            IRNfft+=nchoosek*(gain**k)*IRNfft*exp(-1j*w*k*d)
        IRNadd = np.fft.ifft(IRNfft)
        x=real(IRNadd)
        return Sound(x,samplerate)
    
    @staticmethod
    def irno(delay, gain, niter, duration, samplerate=None, nchannels=1):
        '''
        Returns an IRN_O noise. The iterated ripple noise is obtained many attenuated and
        delayed version of the original broadband noise. 
        For more details: see Yost 1996 or chapter 15 in Hartman Sound Signal Sensation.
        '''
        samplerate = get_samplerate(samplerate)
        noise=Sound.whitenoise(duration)
        splrate=noise.samplerate
        x=array(noise.T)[0]
        IRNadd=np.fft.fft(x)
        Nspl,spl_dur=len(IRNadd),float(1.0/splrate)
        w=2*pi*fftfreq(Nspl,spl_dur)
        d=float(delay)
        for k in range(1,niter+1):
            IRNadd+=(gain**k)*IRNadd*exp(-1j*w*k*d)
        IRNadd = np.fft.ifft(IRNadd)
        x=real(IRNadd)
        return Sound(x, samplerate)

    @staticmethod
    def click(duration, peak=None, samplerate=None, nchannels=1):
        '''
        Returns a click of the given duration.
        
        If ``peak`` is not specified, the amplitude will be 1, otherwise
        ``peak`` refers to the peak dB SPL of the click, according to the
        formula ``28e-6*10**(peak/20.)``.
        '''
        samplerate = get_samplerate(samplerate)
        duration = get_duration(duration,samplerate)
        if peak is not None:
            if not isinstance(peak, dB_type):
                raise dB_error('Peak must be given in dB')
            amplitude = 28e-6*10**(float(peak)/20.)
        else:
            amplitude = 1
        x = amplitude*ones((duration,nchannels))
        return Sound(x, samplerate)
    
    @staticmethod
    def clicks(duration, n, interval, peak=None, samplerate=None, nchannels=1):
        '''
        Returns a series of n clicks (see :func:`click`) separated by interval.
        '''
        oneclick = Sound.click(duration, peak=peak, samplerate=samplerate)
        return oneclick[:interval].repeat(n)

    @staticmethod
    def silence(duration, samplerate=None, nchannels=1):
        '''
        Returns a silent, zero sound for the given duration. Set nchannels to set the number of channels.
        '''
        samplerate = get_samplerate(samplerate)
        duration = get_duration(duration,samplerate)
        x=numpy.zeros((duration,nchannels))
        return Sound(x, samplerate)

    @staticmethod
    def vowel(vowel=None, formants=None, pitch=100*Hz, duration=1*second,
          samplerate=None, nchannels=1):
        '''
        Returns an artifically created spoken vowel sound (following the 
        source-filter model of speech production) with a given ``pitch``.
        
        The vowel can be specified by either providing ``vowel`` as a string
        ('a', 'i' or 'u') or by setting ``formants`` to a sequence of formant
        frequencies.
        
        The returned sound is normalized to a maximum amplitude of 1.
        
        The implementation is based on the MakeVowel function written by Richard
        O. Duda, part of the Auditory Toolbox for Matlab by Malcolm Slaney:
        http://cobweb.ecn.purdue.edu/~malcolm/interval/1998-010/                
        '''    
        
        samplerate = get_samplerate(samplerate)
        duration = get_duration(duration, samplerate)
        
        if not (vowel or formants):
            raise ValueError('Need either a vowel or a list of formants')
        elif (vowel and formants):
            raise ValueError('Cannot use both vowel and formants')
            
        if vowel:
            if vowel == 'a' or vowel == '/a/':
                formants = (730.0*Hz, 1090.0*Hz, 2440.0*Hz)
            elif vowel == 'i' or vowel == '/i/':
                formants = (270.0*Hz, 2290.0*Hz, 3010.0*Hz)
            elif vowel == 'u' or vowel == '/u/':
                formants = (300.0*Hz, 870.0*Hz, 2240.0*Hz)
            else:
                raise ValueError('Unknown vowel: "%s"' % (vowel))            
        
        points = np.arange(0, duration - 1, samplerate / pitch)
            
        indices = np.floor(points).astype(int)
        
        y = np.zeros(duration)
    
        y[indices] = (indices + 1) - points
        y[indices + 1] = points - indices
        
        # model the sound source (periodic glottal excitation)  
        a = np.exp(-250.*Hz * 2 * np.pi / samplerate)
        y = lfilter([1],[1, 0, -a * a], y.copy())
        
        # model the filtering by the vocal tract
        bandwidth = 50.*Hz
        
        for f in formants:
            cft = f / samplerate
            q = f / bandwidth
            rho = np.exp(-np.pi * cft / q)
            theta = 2 * np.pi * cft * np.sqrt(1 - 1/(4.0 * q * q))
            a2 = -2 * rho * np.cos(theta)
            a3 = rho * rho
            y = lfilter([1 + a2 + a3], [1, a2, a3], y.copy()) 
        
        #normalize sound
        data = y / np.max(np.abs(y), axis=0)        
        data.shape = (data.size, 1)
        return Sound(np.tile(data, (nchannels, 1)),  samplerate=samplerate)

    @staticmethod
    def sequence(*args, **kwds):
        '''
        Returns the sequence of sounds in the list sounds joined together
        '''
        samplerate = kwds.pop('samplerate', None)
        if len(kwds):
            raise TypeError('Unexpected keywords to function sequence()')
        sounds = []
        for arg in args:
            if isinstance(arg, (list, tuple)):
                sounds.extend(arg)
            else:
                sounds.append(arg)
        if samplerate is None:
            samplerate = max(s.samplerate for s in sounds)
            rates = unique([int(s.samplerate) for s in sounds])
            if len(rates)>1:
                sounds = tuple(s.resample(samplerate) for s in sounds)
        x = vstack(sounds)
        return Sound(x, samplerate)

    def save(self, filename, normalise=False, samplewidth=2):
        '''
        Save the sound as a WAV.
        
        If the normalise keyword is set to True, the amplitude of the sound will be
        normalised to 1. The samplewidth keyword can be 1 or 2 to save the data as
        8 or 16 bit samples.
        '''
        ext = filename.split('.')[-1].lower()
        if ext=='wav':
            import wave as sndmodule
        elif ext=='aiff' or ext=='aifc':
            import aifc as sndmodule
            raise NotImplementedError('Can only save as wav soundfiles')
        else:
            raise NotImplementedError('Can only save as wav soundfiles')
        
        if samplewidth != 1 and samplewidth != 2:
            raise ValueError('Sample width must be 1 or 2 bytes.')
        
        scale = {2:2 ** 15, 1:2 ** 7-1}[samplewidth]
        if ext=='wav':
            meanval = {2:0, 1:2**7}[samplewidth]
            dtype = {2:int16, 1:uint8}[samplewidth]
            typecode = {2:'h', 1:'B'}[samplewidth]
        else:
            meanval = {2:0, 1:2**7}[samplewidth]
            dtype = {2:int16, 1:uint8}[samplewidth]
            typecode = {2:'h', 1:'B'}[samplewidth]
        w = sndmodule.open(filename, 'wb')
        w.setnchannels(self.nchannels)
        w.setsampwidth(samplewidth)
        w.setframerate(int(self.samplerate))
        x = array(self,copy=True)
        am=amax(x)
        z = zeros(x.shape[0]*self.nchannels, dtype=x.dtype)
        x.shape=(x.shape[0],self.nchannels)
        for i in range(self.nchannels):
            if normalise:
                x[:,i] /= am
            x[:,i] = (x[:,i]) * scale + meanval
            z[i::self.nchannels] = x[::1,i]
        data = array(z, dtype=dtype)
        data = pyarray.array(typecode, data)
        w.writeframes(data.tostring())
        w.close()
    
    @staticmethod
    def load(filename):
        '''
        Load the file given by filename and returns a Sound object. 
        Sound file can be either a .wav or a .aif file.
        '''
        ext = filename.split('.')[-1].lower()
        if ext=='wav':
            import wave as sndmodule
        elif ext=='aif' or ext=='aiff':
            import aifc as sndmodule
        else:
            raise NotImplementedError('Can only load aif or wav soundfiles')
        wav = sndmodule.open(filename, "r")
        nchannels, sampwidth, framerate, nframes, comptype, compname = wav.getparams()
        frames = wav.readframes(nframes * nchannels)
        typecode = {2:'h', 1:'B'}[sampwidth]
        out = frombuffer(frames, dtype=dtype(typecode))
        scale = {2:2 ** 15, 1:2 ** 7-1}[sampwidth]
        meanval = {2:0, 1:2**7}[sampwidth]
        
        data = zeros((nframes, nchannels))
        for i in range(nchannels):
            data[:, i] = out[i::nchannels]
            data[:, i] /= scale
            data[:, i] -= meanval
        
        return Sound(data, samplerate=framerate*Hz)

    def __reduce__(self):
        return (_load_Sound_from_pickle, (asarray(self), float(self.samplerate)))


def _load_Sound_from_pickle(arr, samplerate):
    return Sound(arr, samplerate=samplerate*Hz)

def play(*sounds, **kwds):
    '''
    Plays a sound or sequence of sounds. For example::
    
        play(sound)
        play(sound1, sound2)
        play([sound1, sound2, sound3])
        
    If ``normalise=True``, the sequence of sounds will be normalised to the
    maximum range (-1 to 1), and if ``sleep=True`` the function will wait
    until the sounds have finished playing before returning.
    '''
    normalise = kwds.pop('normalise', False)
    sleep = kwds.pop('sleep', False)
    if len(kwds):
        raise TypeError('Unexpected keyword arguments to function play()')
    sound = sequence(*sounds)
    sound.play(normalise=normalise, sleep=sleep)
play.__doc__ = Sound.play.__doc__

def savesound(sound, filename, normalise=False, samplewidth=2):
    sound.save(filename, normalise=normalise, samplewidth=samplewidth)
savesound.__doc__ = Sound.save.__doc__

def get_duration(duration,samplerate):
    if not isinstance(duration, int):
        duration = int(rint(duration * samplerate))
    return duration


whitenoise = Sound.whitenoise
powerlawnoise = Sound.powerlawnoise
pinknoise = Sound.pinknoise
brownnoise = Sound.brownnoise
irns = Sound.irns
irno = Sound.irno
tone = Sound.tone
harmoniccomplex = Sound.harmoniccomplex
click = Sound.click
clicks = Sound.clicks
silence = Sound.silence
sequence = Sound.sequence
vowel = Sound.vowel
loadsound = Sound.load



from brian import *

def test_delay_connect_with_subgroups():
    G = NeuronGroup(4, 'V:1')
    # test connect method
    C1 = Connection(G, G, 'V', delay=True, structure='dense')
    C2 = Connection(G, G, 'V', delay=True, structure='sparse')
    C3 = Connection(G, G, 'V', delay=True, structure='dynamic')
    for i, C in enumerate([C1, C2, C3]):
        C.connect(G[0:2], G[0:2], W=ones((2, 2)), delay=1)
        C.connect(G[0:2], G[2:4], W=ones((2, 2)), delay=(2.1, 2.2))
        C.connect(G[2:4], G[0:2], W=ones((2, 2)), delay=lambda:4)
        C.connect(G[2:4], G[2:4], W=ones((2, 2)), delay=lambda i, j:i + 10 * j)
        assert (C.W.todense() == 1).all(), 'Problem with connection C' + str(i + 1)
        D = C.delayvec.todense()
        assert (array(D, dtype=int) == array([[1, 1, 2, 2],
                                            [1, 1, 2, 2],
                                            [4, 4, 0, 10],
                                            [4, 4, 1, 11]], dtype=int)).all(), 'Problem with connection C' + str(i + 1)
    # test connect_random method
    C1 = Connection(G, G, 'V', delay=True, structure='dense')
    C2 = Connection(G, G, 'V', delay=True, structure='sparse')
    C3 = Connection(G, G, 'V', delay=True, structure='dynamic')
    for i, C in enumerate([C1, C2, C3]):
        C.connect_random(G[0:2], G[0:2], p=1, weight=1, delay=1)
        C.connect_random(G[0:2], G[2:4], p=1, weight=1, delay=(2.1, 2.2))
        C.connect_random(G[2:4], G[0:2], p=1, weight=1, delay=lambda:4)
        C.connect_random(G[2:4], G[2:4], p=1, weight=1, delay=lambda i, j:i + 10 * j)
        assert (C.W.todense() == 1).all(), 'Problem with connection C' + str(i + 1)
        D = C.delayvec.todense()
        assert (array(D, dtype=int) == array([[1, 1, 2, 2],
                                            [1, 1, 2, 2],
                                            [4, 4, 0, 10],
                                            [4, 4, 1, 11]], dtype=int)).all(), 'Problem with connection C' + str(i + 1)
    # test connect_full method
    C1 = Connection(G, G, 'V', delay=True, structure='dense')
    C2 = Connection(G, G, 'V', delay=True, structure='sparse')
    C3 = Connection(G, G, 'V', delay=True, structure='dynamic')
    for i, C in enumerate([C1, C2, C3]):
        C.connect_full(G[0:2], G[0:2], weight=1, delay=1)
        C.connect_full(G[0:2], G[2:4], weight=1, delay=(2.1, 2.2))
        C.connect_full(G[2:4], G[0:2], weight=1, delay=lambda:4)
        C.connect_full(G[2:4], G[2:4], weight=1, delay=lambda i, j:i + 10 * j)
        assert (C.W.todense() == 1).all(), 'Problem with connection C' + str(i + 1)
        D = C.delayvec.todense()
        assert (array(D, dtype=int) == array([[1, 1, 2, 2],
                                            [1, 1, 2, 2],
                                            [4, 4, 0, 10],
                                            [4, 4, 1, 11]], dtype=int)).all(), 'Problem with connection C' + str(i + 1)
    # test connect_one_to_one method
    C1 = Connection(G, G, 'V', delay=True, structure='dense')
    C2 = Connection(G, G, 'V', delay=True, structure='sparse')
    C3 = Connection(G, G, 'V', delay=True, structure='dynamic')
    for i, C in enumerate([C1, C2, C3]):
        C.connect_one_to_one(G[0:2], G[0:2], weight=1, delay=1)
        C.connect_one_to_one(G[0:2], G[2:4], weight=1, delay=(2.1, 2.2))
        C.connect_one_to_one(G[2:4], G[0:2], weight=1, delay=lambda:4)
        C.connect_one_to_one(G[2:4], G[2:4], weight=1, delay=lambda i, j:i + 10 * j)
        D = C.delayvec.todense()
        assert (array(D, dtype=int) == array([[1, 0, 2, 0],
                                            [0, 1, 0, 2],
                                            [4, 0, 0, 0],
                                            [0, 4, 0, 11]], dtype=int)).all(), 'Problem with connection C' + str(i + 1)

def test_connect_with_subgroups():
    G = NeuronGroup(4, 'V:1')
    # test connect method
    C1 = Connection(G, G, 'V', structure='dense')
    C2 = Connection(G, G, 'V', structure='sparse')
    C3 = Connection(G, G, 'V', structure='dynamic')
    for i, C in enumerate([C1, C2, C3]):
        C.connect(G[0:2], G[0:2], W=ones((2, 2)))
        C.connect(G[0:2], G[2:4], W=2 * ones((2, 2)))
        C.connect(G[2:4], G[0:2], W=4 * ones((2, 2)))
        C.connect(G[2:4], G[2:4], W=array([[0, 10], [1, 11]]))
        W = C.W.todense()
        assert (array(W, dtype=int) == array([[1, 1, 2, 2],
                                            [1, 1, 2, 2],
                                            [4, 4, 0, 10],
                                            [4, 4, 1, 11]], dtype=int)).all(), 'Problem with connection C' + str(i + 1)
    # test connect_random method
    C1 = Connection(G, G, 'V', structure='dense')
    C2 = Connection(G, G, 'V', structure='sparse')
    C3 = Connection(G, G, 'V', structure='dynamic')
    for i, C in enumerate([C1, C2, C3]):
        C.connect_random(G[0:2], G[0:2], p=1, weight=1)
        C.connect_random(G[2:4], G[2:4], p=1, weight=lambda i, j:i + 10 * j)
        W = C.W.todense()
        assert (array(W, dtype=int) == array([[1, 1, 0, 0],
                                            [1, 1, 0, 0],
                                            [0, 0, 0, 10],
                                            [0, 0, 1, 11]], dtype=int)).all(), 'Problem with connection C' + str(i + 1)
    # test connect_full method
    C1 = Connection(G, G, 'V', structure='dense')
    C2 = Connection(G, G, 'V', structure='sparse')
    C3 = Connection(G, G, 'V', structure='dynamic')
    for i, C in enumerate([C1, C2, C3]):
        C.connect_full(G[0:2], G[0:2], weight=1)
        C.connect_full(G[2:4], G[2:4], weight=lambda i, j:i + 10 * j)
        W = C.W.todense()
        assert (array(W, dtype=int) == array([[1, 1, 0, 0],
                                            [1, 1, 0, 0],
                                            [0, 0, 0, 10],
                                            [0, 0, 1, 11]], dtype=int)).all(), 'Problem with connection C' + str(i + 1)
    # test connect_one_to_one method
    C1 = Connection(G, G, 'V', structure='dense')
    C2 = Connection(G, G, 'V', structure='sparse')
    C3 = Connection(G, G, 'V', structure='dynamic')
    for i, C in enumerate([C1, C2, C3]):
        C.connect_one_to_one(G[0:2], G[0:2], weight=1)
        W = C.W.todense()
        assert (array(W, dtype=int) == array([[1, 0, 0, 0],
                                            [0, 1, 0, 0],
                                            [0, 0, 0, 0],
                                            [0, 0, 0, 0]], dtype=int)).all(), 'Problem with connection C' + str(i + 1)

if __name__ == '__main__':
    test_delay_connect_with_subgroups()
    test_connect_with_subgroups()



'''
Tutorial 1c: Making some activity
*********************************

In the previous part of the tutorial we found that each neuron
was producing only one spike. In this part, we alter the model so
that some more spikes will be generated. What we'll do is alter
the resting potential ``El`` so that it is above threshold, this
will ensure that some spikes are generated. The first few
lines remain the same: 
'''
from brian import *

tau = 20 * msecond        # membrane time constant
Vt = -50 * mvolt          # spike threshold
Vr = -60 * mvolt          # reset value
'''
But we change the resting potential to -49 mV, just above the
spike threshold:
'''
El = -49 * mvolt          # resting potential (same as the reset)
'''
And then continue as before:
'''
G = NeuronGroup(N=40, model='dV/dt = -(V-El)/tau : volt',
              threshold=Vt, reset=Vr)

M = SpikeMonitor(G)

run(1 * second)

print M.nspikes
'''
Running this program gives the output ``840``. That's because
every neuron starts at the same initial value and proceeds
deterministically, so that each neuron fires at exactly the
same time, in total 21 times during the 1s of the run.

In the next part, we'll introduce a random element into the
behaviour of the network.

Exercises
~~~~~~~~~

1. Try varying the parameters and seeing how the number of
   spikes generated varies.
2. Solve the differential equation by hand and compute a
   formula for the number of spikes generated. Compare this
   with the program output and thereby partially verify it.
   (Hint: each neuron starts at above the threshold and so
   fires a spike immediately.)

Solution
~~~~~~~~

Solving the differential equation gives:

    V = El + (Vr-El) exp (-t/tau)

Setting V=Vt at time t gives:

    t = tau log( (Vr-El) / (Vt-El) )

If the simulator runs for time T, and fires a spike immediately
at the beginning of the run it will then generate n spikes,
where:

    n = [T/t] + 1

If you have m neurons all doing the same thing, you get nm
spikes. This calculation with the parameters above gives:

    t = 48.0 ms
    n = 21
    nm = 840

As predicted.
'''



from base import *
from sparsematrix import *
from connectionmatrix import *

__all__ = [
         'ConstructionMatrix',
         'SparseConstructionMatrix',
         'DenseConstructionMatrix',
         'DynamicConstructionMatrix',
         'construction_matrix_register',
         ]

class ConstructionMatrix(object):
    '''
    Base class for construction matrices
    
    A construction matrix is used to initialise and build connection matrices.
    A ``ConstructionMatrix`` class has to implement a method
    ``connection_matrix(*args, **kwds)`` which returns a :class:`ConnectionMatrix`
    object of the appropriate type.
    '''
    def connection_matrix(self, *args, **kwds):
        return NotImplemented


class DenseConstructionMatrix(ConstructionMatrix, numpy.ndarray):
    '''
    Dense construction matrix. Essentially just numpy.ndarray.
    
    The ``connection_matrix`` method returns a :class:`DenseConnectionMatrix`
    object.
    
    The ``__setitem__`` method is overloaded so that you can set values with
    a sparse matrix.
    '''
    def __init__(self, val, **kwds):
        self[:] = 0
        self.init_kwds = kwds

    def connection_matrix(self, **additional_kwds):
        self.init_kwds.update(additional_kwds)
        kwds = self.init_kwds
        return DenseConnectionMatrix(self, **kwds)

    def __setitem__(self, index, W):
        # Make it work for sparse matrices
        #if isinstance(W,sparse.spmatrix):
        if isinstance(W, sparse.spmatrix):
            ndarray.__setitem__(self, index, W.todense())
        else:
            ndarray.__setitem__(self, index, W)

    def todense(self):
        return asarray(self)

class SparseConstructionMatrix(ConstructionMatrix, SparseMatrix):
    '''
    SparseConstructionMatrix is converted to SparseConnectionMatrix.
    '''
    def __init__(self, arg, **kwds):
        SparseMatrix.__init__(self, arg)
        self.init_kwds = kwds

    def connection_matrix(self, **additional_kwds):
        self.init_kwds.update(additional_kwds)
        return SparseConnectionMatrix(self, **self.init_kwds)


class DynamicConstructionMatrix(ConstructionMatrix, SparseMatrix):
    '''
    DynamicConstructionMatrix is converted to DynamicConnectionMatrix.
    '''
    def __init__(self, arg, **kwds):
        SparseMatrix.__init__(self, arg)
        self.init_kwds = kwds

    def connection_matrix(self, **additional_kwds):
        self.init_kwds.update(additional_kwds)
        return DynamicConnectionMatrix(self, **self.init_kwds)

# this is used to look up str->class conversions for structure=... keyword
construction_matrix_register = {
        'dense':DenseConstructionMatrix,
        'sparse':SparseConstructionMatrix,
        'dynamic':DynamicConstructionMatrix,
        }



from brian import *
from brian.experimental.neuromorphic import *

defaultclock.dt = 1*ms
runtime = 100*ms
g = PoissonGroup(100, 100*Hz)
#g = SpikeGeneratorGroup(1, [(0,t*ms) for t in range(int(runtime/defaultclock.dt))])

Maer = AERSpikeMonitor(g, './dummy.aedat')


run(100*ms)

Maer.close_file()


addr, timestamps = load_AER('./dummy.aedat')
if len(addr) == len(M.spikes):
    print 'looks good'
else:
    print 'glub'
    print 'addr, M.spikes',len(addr),len(M.spikes)

# Check for NewSpikeGeneratorGroup
N = max(addr)
group = NewSpikeGeneratorGroup(N, (addr, timestamps), time_unit = 1*usecond)

plot(timestamps, addr, '.')

show()



from base import *
from connection import *

__all__ = [
         'IdentityConnection',
         'MultiConnection',
         ]

class IdentityConnection(Connection):
    '''
    A :class:`Connection` between two groups of the same size, where neuron ``i`` in the
    source group is connected to neuron ``i`` in the target group.
    
    Initialised with arguments:
    
    ``source``, ``target``
        The source and target :class:`NeuronGroup` objects.
    ``state``
        The target state variable.
    ``weight``
        The weight of the synapse, must be a scalar.
    ``delay``
        Only homogeneous delays are allowed.
    
    The benefit of this class is that it has no storage requirements and is optimised for
    this special case.
    '''
    @check_units(delay=second)
    def __init__(self, source, target, state=0, weight=1, delay=0 * msecond):
        if (len(source) != len(target)):
            raise AttributeError, 'The connected (sub)groups must have the same size.'
        self.source = source # pointer to source group
        self.target = target # pointer to target group
        if type(state) == types.StringType: # named state variable
            self.nstate = target.get_var_index(state)
        else:
            self.nstate = state # target state index
        self.W = float(weight) # weight
        source.set_max_delay(delay)
        self.delay = int(delay / source.clock.dt) # Synaptic delay in time bins

    def propagate(self, spikes):
        '''
        Propagates the spikes to the target.
        '''
        self.target._S[self.nstate, spikes] += self.W

    def compress(self):
        pass


class MultiConnection(Connection):
    '''
    A hub for multiple connections with a common source group.
    '''
    def __init__(self, source, connections=[]):
        self.source = source
        self.connections = connections
        self.iscompressed = False
        self.delay = connections[0].delay

    def propagate(self, spikes):
        '''
        Propagates the spikes to the targets.
        '''
        for C in self.connections:
            C.propagate(spikes)

    def compress(self):
        if not self.iscompressed:
            for C in self.connections:
                C.compress()
            self.iscompressed = True

    def reinit(self):
        for C in self.connections:
            # this does nothing for normal connections but is important for
            # SpikeMonitors
            C.reinit()   


'''
Jeffress model, adapted with spiking neuron models.
A sound source (white noise) is moving around the head.
Delay differences between the two ears are used to determine the azimuth of the source.
Delays are mapped to a neural place code using delay lines (each neuron receives input
from both ears, with different delays).

Romain Brette
'''
from brian import *

defaultclock.dt = .02 * ms
dt = defaultclock.dt

# Sound
sound = TimedArray(10 * randn(50000)) # white noise

# Ears and sound motion around the head (constant angular speed)
sound_speed = 300 * metre / second
interaural_distance = 20 * cm # big head!
max_delay = interaural_distance / sound_speed
print "Maximum interaural delay:", max_delay
angular_speed = 2 * pi * radian / second # 1 turn/second
tau_ear = 1 * ms
sigma_ear = .1
eqs_ears = '''
dx/dt=(sound(t-delay)-x)/tau_ear+sigma_ear*(2./tau_ear)**.5*xi : 1
delay=distance*sin(theta) : second
distance : second # distance to the centre of the head in time units
dtheta/dt=angular_speed : radian
'''
ears = NeuronGroup(2, model=eqs_ears, threshold=1, reset=0, refractory=2.5 * ms)
ears.distance = [-.5 * max_delay, .5 * max_delay]
traces = StateMonitor(ears, 'x', record=True)

# Coincidence detectors
N = 300
tau = 1 * ms
sigma = .1
eqs_neurons = '''
dv/dt=-v/tau+sigma*(2./tau)**.5*xi : 1
'''
neurons = NeuronGroup(N, model=eqs_neurons, threshold=1, reset=0)
synapses = Connection(ears, neurons, 'v', structure='dense', delay=True, max_delay=1.1 * max_delay)
synapses.connect_full(ears, neurons, weight=.5)
synapses.delay[0, :] = linspace(0 * ms, 1.1 * max_delay, N)
synapses.delay[1, :] = linspace(0 * ms, 1.1 * max_delay, N)[::-1]
spikes = SpikeMonitor(neurons)

run(1000 * ms)
raster_plot(spikes)
show()



from brian import *
set_global_preferences(usenewbrianhears=True)
from brian.hears import *

# Load database
hrtfdb = IRCAM_LISTEN(r'D:\HRTF\IRCAM')
hrtfset = hrtfdb.load_subject(1002)
# Select only the horizontal plane
hrtfset = hrtfset.subset(lambda elev: elev==0)
# Set up a filterbank
sound = whitenoise(10*ms)
fb = hrtfset.filterbank(sound)
# Extract the filtered response and plot
img = fb.buffer_fetch(0, len(sound)).T
img_left = img[:img.shape[0]/2, :]
img_right = img[img.shape[0]/2:, :]
subplot(121)
imshow(img_left, origin='lower left', aspect='auto',
       extent=(0, sound.duration/ms, 0, 360))
xlabel('Time (ms)')
ylabel('Azimuth')
title('Left ear')
subplot(122)
imshow(img_right, origin='lower left', aspect='auto',
       extent=(0, sound.duration/ms, 0, 360))
xlabel('Time (ms)')
ylabel('Azimuth')
title('Right ear')
show()



"""
Loads an AER .dat file and plays it in Brian

This works (more or less).

The main bottleneck is creating the SpikeGeneratorGroup,
specifically the gather() method.
"""
from brian import *
from brian.experimental.neuromorphic import *
from time import time

#defaultclock.dt=1*ms

path=r'C:\Users\Romain\Desktop\jaerSampleData\DVS128'
filename=r'\Tmpdiff128-2006-02-03T14-39-45-0800-0 tobi eye.dat'

addr,timestamp=load_AER(path+filename)
print len(addr),"events"
#print "Timestamps sorted:",all(diff(timestamp)>=0) # Check if sorted
x,y,pol=extract_DVS_event(addr)
spiketimes=array((y,timestamp*1e-6)).T

t0=time()
#P=SpikeGeneratorGroup(128,spiketimes,gather=False)
# True: 3.66 (3.55 for the bottleneck) + 1.45
# False: 0.04 + 0.75 (Running is also faster!!)
P=SpikeQueue(128,timestamp*1e-6,y,check_sorted=False)
# 0.88 + .47 (Running is faster, but construction is slower)
t00=time()
print "Group creation:",t00-t0
M=SpikeMonitor(P)

t0=time()
run(1*second)
t00=time()
print "Running time:",t00-t0

# Checking
spikes=array(zip(*M.spikes)[1])
print timestamp[:10],spikes[:10]*1e6

raster_plot(M)
show()



from brian import *
from brian.experimental.codegen.reset import *

eqs = Equations('''
#dV/dt = -V/(100*second) : volt
V : volt
Vt : volt
''')

dVt = 2 * mV

reset = '''
V = 0*volt
Vt += dVt
'''

print '**** C CODE *****'
print generate_c_reset(eqs, reset)

print '**** PYTHON CODE ****'
print generate_python_reset(eqs, reset)

GP = NeuronGroup(10, eqs, threshold=1 * volt,
                reset=PythonReset(reset))
GC = NeuronGroup(10, eqs, threshold=1 * volt,
                reset=CReset(reset))

GP.V = 0.5
GP.Vt = 0.1
GP.V[[2, 4, 8]] = 2
GC.V = 0.5
GC.Vt = 0.1
GC.V[[2, 4, 8]] = 2
run(1 * ms)
print 'GP.V  =', GP.V
print 'GP.Vt =', GP.Vt
print 'GC.V  =', GC.V
print 'GC.Vt =', GC.Vt



from brian import *
from scipy import signal, weave, random
from ..bufferable import Bufferable
from operator import isSequenceType
from __builtin__ import all

__all__ = ['Filterbank',
           'RestructureFilterbank',
                'Repeat', 'Tile', 'Join', 'Interleave',
           'FunctionFilterbank',
           'SumFilterbank',
           'DoNothingFilterbank',
           'ControlFilterbank',
           'CombinedFilterbank',
           ]

class Filterbank(Bufferable):
    '''
    Generalised filterbank object
    
    **Documentation common to all filterbanks**
    
    Filterbanks all share a few basic attributes:
    
    .. autoattribute:: source
    
    .. attribute:: nchannels
    
        The number of channels.
        
    .. attribute:: samplerate
    
        The sample rate.
        
    .. autoattribute:: duration

    To process the output of a filterbank, the following method can be used:
    
    .. automethod:: process
    
    Alternatively, the buffer interface can be used, which is described in
    more detail below.
    
    Filterbank also defines arithmetical operations for +, -, ``*``, / where the other
    operand can be a filterbank or scalar.
    
    **Details on the class**
    
    This class is a base class not designed to be instantiated. A Filterbank
    object should define the interface of :class:`Bufferable`, as well as
    defining a ``source`` attribute. This is normally a :class:`Bufferable`
    object, but could be an iterable of sources (for example, for filterbanks
    that mix or add multiple inputs).
    
    The ``buffer_fetch_next(samples)`` method has a default implementation
    that fetches the next input, and calls the ``buffer_apply(input)``
    method on it, which can be overridden by a derived class. This is typically
    the easiest way to implement a new filterbank. Filterbanks with multiple
    sources will need to override this default implementation.
    
    There is a default ``__init__`` method that can be called by a derived class
    that sets the ``source``, ``nchannels`` and ``samplerate`` from that of the
    ``source`` object. For multiple sources, the default implementation will
    check that each source has the same number of channels and samplerate and
    will raise an error if not.
    
    There is a default ``buffer_init()`` method that calls ``buffer_init()`` on
    the ``source`` (or list of sources).
        
    **Example of deriving a class**
    
    The following class takes N input channels and sums them to a single output
    channel::
    
        class AccumulateFilterbank(Filterbank):
            def __init__(self, source):
                Filterbank.__init__(self, source)
                self.nchannels = 1
            def buffer_apply(self, input):
                return reshape(sum(input, axis=1), (input.shape[0], 1))
            
    Note that the default ``Filterbank.__init__`` will set the number of
    channels equal to the number of source channels, but we want to change it
    to have a single output channel. We use the ``buffer_apply`` method which
    automatically handles the efficient cacheing of the buffer for us. The
    method receives the array ``input`` which has shape ``(bufsize, nchannels)``
    and sums over the channels (``axis=1``). It's important to reshape the
    output so that it has shape ``(bufsize, outputnchannels)`` so that it can
    be used as the input to subsequent filterbanks.
    '''
       
    def __init__(self, source):
        if isinstance(source, Bufferable):
            self.source = source
            self.nchannels = source.nchannels
            self.samplerate = source.samplerate
        else:
            self.nchannels = source[0].nchannels
            self.samplerate = source[0].samplerate
            for s in source:
                if s.nchannels!=self.nchannels:
                    raise ValueError('All sources must have the same number of channels.')
                if int(s.samplerate)!=int(self.samplerate):
                    raise ValueError('All sources must have the same samplerate.')
            self.source = source

    def change_source(self, source):
        if not hasattr(self, '_source') or self._source is None:
            self._source = source
            return
        if isinstance(source, tuple):
            for s in source:
                if int(s.samplerate)!=int(self.samplerate):
                    raise ValueError('source samplerate is wrong.')
            for news, olds in zip(source, self._source):
                if news.nchannels!=olds.nchannels:
                    raise ValueError('New sources have different numbers of channels to old sources.')
            self._source = source
            return
        if source.nchannels==self.nchannels:
            self._source = source
            return
        if source.nchannels==1:
            self._source = Repeat(source, self.nchannels)
        else:
            raise ValueError('New source must have the same number of channels as old source.')

    source = property(fget=lambda self:self._source,
                      fset=lambda self, source:self.change_source(source),
                      doc='''
        The source of the filterbank, a :class:`Bufferable` object, e.g. another
        :class:`Filterbank` or a :class:`Sound`. It can also be a tuple of 
        sources. Can be changed after the object
        is created, although note that for some filterbanks this may cause
        problems if they do make assumptions about the input based on the first
        source object they were passed. If this is causing problems, you can
        insert a dummy filterbank (:class:`DoNothingFilterbank`) which is
        guaranteed to work if you change the source.
        ''')
    
    def get_duration(self):
        if hasattr(self, '_duration'):
            return self._duration
        else:
            source = self.source
            if isinstance(source, Bufferable):
                source = [source]
            try:
                durations = [s.duration for s in source]
                duration = max(durations)
                return duration
            except KeyError:
                raise KeyError('Cannot compute duration from sources.')

    def set_duration(self, duration):
        self._duration = duration
    
    duration = property(fget=get_duration, fset=set_duration, doc='''
        The duration of the filterbank. If it is not specified by the user, it
        is computed by finding the maximum of its source durations. If these are
        not specified a :class:`KeyError` will be raised (for example, using
        :class:`OnlineSound` as a source).
        ''')

    def process(self, func=None, duration=None, buffersize=32):
        '''
        Returns the output of the filterbank for the given duration.
        
        ``func``
            If a function is specified, it should be a function of one or two
            arguments that will be called on each filtered buffered segment
            (of shape ``(buffersize, nchannels)`` in order. If the function has
            one argument, the argument should be buffered segment. If it has
            two arguments, the second argument is the value returned by the
            previous application of the function (or 0 for the first
            application). In this case, the method will return the final
            value returned by the function. See example below.
        ``duration=None``
            The length of time (in seconds) or number of samples to process.
            If no ``func`` is specified, the method will return an array of shape
            ``(duration, nchannels)`` with the filtered outputs. Note that in
            many cases, this will be too large to fit in memory, in which you
            will want to process the filtered outputs online, by providing
            a function ``func`` (see example below). If no duration is specified,
            the maximum duration of the inputs to the filterbank will be used,
            or an error raised if they do not have durations (e.g. in the case
            of :class:`OnlineSound`).
        ``buffersize=32``
            The size of the buffered segments to fetch, as a length of time or
            number of samples. 32 samples typically gives reasonably good
            performance.
            
        For example, to compute the RMS of each channel in a filterbank, you
        would do::
        
            def sum_of_squares(input, running_sum_of_squares):
                return running_sum_of_squares+sum(input**2, axis=0)
            rms = sqrt(fb.process(sum_of_squares)/nsamples)
        '''
        if duration is None:
            duration = self.duration
        if not isinstance(duration, int):
            duration = int(duration*self.samplerate)
        if not isinstance(buffersize, int):
            buffersize = int(buffersize*self.samplerate)
        self.buffer_init()
        endpoints = hstack((arange(0, duration, buffersize), duration))
        zendpoints = zip(endpoints[:-1], endpoints[1:])
        #sizes = diff(endpoints)
        if func is None:
            return vstack(tuple(self.buffer_fetch(start, end) for start, end in zendpoints))
        else:
            if func.func_code.co_argcount==1:
                for start, end in zendpoints:
                    func(self.buffer_fetch(start, end))
            else:
                runningval = 0
                for start, end in zendpoints:
                    runningval = func(self.buffer_fetch(start, end), runningval)
                return runningval

    def buffer_init(self):
        Bufferable.buffer_init(self)
        if isinstance(self.source, Bufferable):
            self.source.buffer_init()
        else:
            for s in self.source:
                s.buffer_init()
        self.next_sample = 0

    def buffer_apply(self, input):
        raise NotImplementedError

    def buffer_fetch_next(self, samples):
        start = self.next_sample
        self.next_sample += samples
        end = start+samples
        input = self.source.buffer_fetch(start, end)
        return self.buffer_apply(input)
    
    def __add__ (self, other):
        if isinstance(other, Bufferable):
            return SumFilterbank((self, other))
        else:
            func = lambda x: other+x
            return FunctionFilterbank(self, func)
    __radd__ = __add__
        
    def __sub__ (self, other):
        if isinstance(other, Bufferable):
            return SumFilterbank((self, other), (1, -1))
        else:
            func = lambda x: x-other
            return FunctionFilterbank(self, func)
        
    def __rsub__ (self, other):
        # Note that __rsub__ should return other-self
        if isinstance(other, Bufferable):
            return SumFilterbank((self, other), (-1, 1))
        else:
            func = lambda x: other-x
            return FunctionFilterbank(self, func)

    def __mul__(self, other):
        if isinstance(other, Bufferable):
            func = lambda x, y: x*y
            return FunctionFilterbank((self, other), func)
        else:
            func = lambda x: x*other
            return FunctionFilterbank(self, func)        
    __rmul__ = __mul__

    def __div__(self, other):
        if isinstance(other, Bufferable):
            func = lambda x, y: x/y
            return FunctionFilterbank((self, other), func)
        else:
            func = lambda x: x/other
            return FunctionFilterbank(self, func)        

    def __rdiv__(self, other):
        # Note __rdiv__ returns other/self
        if isinstance(other, Bufferable):
            func = lambda x, y: x/y
            return FunctionFilterbank((other, self), func)
        else:
            func = lambda x: other/x
            return FunctionFilterbank(self, func)        


class RestructureFilterbank(Filterbank):
    '''
    Filterbank used to restructure channels, including repeating and interleaving.
    
    **Standard forms of usage:**
    
    Repeat mono source N times::
    
        RestructureFilterbank(source, N)
        
    For a stereo source, N copies of the left channel followed by N copies of
    the right channel::
    
        RestructureFilterbank(source, N)
        
    For a stereo source, N copies of the channels tiled as LRLRLR...LR::
    
        RestructureFilterbank(source, numtile=N)
        
    For two stereo sources AB and CD, join them together in serial to form the
    output channels in order ABCD::
    
        RestructureFilterbank((AB, CD))
        
    For two stereo sources AB and CD, join them together interleaved to form
    the output channels in order ACBD::
    
        RestructureFilterbank((AB, CD), type='interleave')
        
    These arguments can also be combined together, for example to AB and CD
    into output channels AABBCCDDAABBCCDDAABBCCDD::
    
        RestructureFilterbank((AB, CD), 2, 'serial', 3)
        
    The three arguments are the number of repeats before joining, the joining
    type ('serial' or 'interleave') and the number of tilings after joining.
    See below for details.
    
    **Initialise arguments:**
    
    ``source``
        Input source or list of sources.
    ``numrepeat=1``
        Number of times each channel in each of the input sources is repeated
        before mixing the source channels. For example, with repeat=2 an input
        source with channels ``AB`` will be repeated to form ``AABB``
    ``type='serial'``
        The method for joining the source channels, the options are ``'serial'``
        to join the channels in series, or ``'interleave'`` to interleave them.
        In the case of ``'interleave'``, each source must have the same number
        of channels. An example of serial, if the input sources are ``abc``
        and ``def`` the output would be ``abcdef``. For interleave, the output
        would be ``adbecf``.
    ``numtile=1``
        The number of times the joined channels are tiled, so if the joined
        channels are ``ABC`` and ``numtile=3`` the output will be ``ABCABCABC``.
    ``indexmapping=None``
        Instead of specifying the restructuring via ``numrepeat, type, numtile``
        you can directly give the mapping of input indices to output indices.
        So for a single stereo source input, ``indexmapping=[1,0]`` would
        reverse left and right. Similarly, with two mono sources,
        ``indexmapping=[1,0]`` would have channel 0 of the output correspond to
        source 1 and channel 1 of the output corresponding to source 0. This is
        because the indices are counted in order of channels starting from the
        first source and continuing to the last. For example, suppose you had
        two sources, each consisting of a stereo sound, say source 0 was
        ``AB`` and source 1 was ``CD`` then ``indexmapping=[1, 0, 3, 2]`` would
        swap the left and right of each source, but leave the order of the
        sources the same, i.e. the output would be ``BADC``.        
    '''
    def __init__(self, source, numrepeat=1, type='serial', numtile=1,
                 indexmapping=None):
        self._has_been_optimised = False
        self._reinit(source, numrepeat, type, numtile, indexmapping)
    
    def _do_reinit(self):
        self._reinit(*self._original_init_arguments)
        if self._has_been_optimised:
            self._optimisation_target._do_reinit()
    
    def _reinit(self, source, numrepeat, type, numtile, indexmapping):
        self._original_init_arguments = (source, numrepeat, type, numtile, indexmapping)
        if isinstance(source, Bufferable):
            source = (source,)
        if indexmapping is None:
            nchannels = array([s.nchannels for s in source])
            idx = hstack(([0], cumsum(nchannels)))
            I = [arange(start, stop) for start, stop in zip(idx[:-1], idx[1:])]
            I = tuple(repeat(i, numrepeat) for i in I)
            if type=='serial':
                indexmapping = hstack(I)
            elif type=='interleave':
                if len(unique(nchannels))!=1:
                    raise ValueError('For interleaving, all inputs must have an equal number of channels.')
                I0 = len(I[0])
                indexmapping = zeros(I0*len(I), dtype=int)
                for j, i in enumerate(I):
                    indexmapping[j::len(I)] = i
            else:
                raise ValueError('Type must be "serial" or "interleave"')
            indexmapping = tile(indexmapping, numtile)
        if not isinstance(indexmapping, ndarray):
            indexmapping = array(indexmapping, dtype=int)
        # optimisation to reduce multiple RestructureFilterbanks into a single
        # one, by collating the sources and reconstructing the indexmapping
        # from the individual indexmappings
        if all(isinstance(s, RestructureFilterbank) for s in source):
            newsource = ()
            newsourcesizes = ()
            for s in source:
                s._has_been_optimised = True
                s._optimisation_target = self
                newsource += s.source
                inputsourcesize = sum(inpsource.nchannels for inpsource in s.source)
                newsourcesizes += (inputsourcesize,)
            newsourcesizes = array(newsourcesizes)
            newsourceoffsets = hstack((0, cumsum(newsourcesizes)))
            new_indexmapping = zeros_like(indexmapping)
            sourcesizes = array(tuple(s.nchannels for s in source))
            sourceoffsets = hstack((0, cumsum(sourcesizes)))
            # gives the index of the source of each element of indexmapping
            sourceindices = digitize(indexmapping, cumsum(sourcesizes))
            for i in xrange(len(indexmapping)):
                source_index = sourceindices[i]
                s = source[source_index]
                relative_index = indexmapping[i]-sourceoffsets[source_index]
                source_relative_index = s.indexmapping[relative_index]
                new_index = source_relative_index+newsourceoffsets[source_index]
                new_indexmapping[i] = new_index
            source = newsource
            indexmapping = new_indexmapping
                
        self.indexmapping = indexmapping
        self.nchannels = len(indexmapping)
        self.samplerate = source[0].samplerate
        for s in source:
            if int(s.samplerate)!=int(self.samplerate):
                raise ValueError('All sources must have the same samplerate.')
        self._source = source

    def buffer_fetch_next(self, samples):
        start = self.next_sample
        self.next_sample += samples
        end = start+samples
        inputs = tuple(s.buffer_fetch(start, end) for s in self.source)
        input = hstack(inputs)
        input = input[:, self.indexmapping]
        return input

    def change_source(self, source):
        if not hasattr(self, '_source') or self._source is None:
            self._source = source
            return
        oldsource, numrepeat, type, numtile, indexmapping = self._original_init_arguments
        self._original_init_arguments = source, numrepeat, type, numtile, indexmapping
        self._do_reinit()
#        self._reinit(source, numrepeat, type, numtile, indexmapping)
#        if self._has_been_optimised:
#            target = self._optimisation_target
#            target._reinit(*target._original_init_arguments)

class Repeat(RestructureFilterbank):
    '''
    Filterbank that repeats each channel from its input, e.g. with 3 repeats
    channels ABC would map to AAABBBCCC.
    '''
    def __init__(self, source, numrepeat):
        RestructureFilterbank.__init__(self, source, numrepeat)

class Tile(RestructureFilterbank):
    '''
    Filterbank that tiles the channels from its input, e.g. with 3 tiles
    channels ABC would map to ABCABCABC.
    '''
    def __init__(self, source, numtile):
        RestructureFilterbank.__init__(self, source, numtile=numtile)
        
class Join(RestructureFilterbank):
    '''
    Filterbank that joins the channels of its inputs in series, e.g. with two
    input sources with channels AB and CD respectively, the output would have
    channels ABCD. You can initialise with multiple sources separated by
    commas, or by passing a list of sources.
    '''
    def __init__(self, *sources):
        source = []
        for s in sources:
            if isinstance(s, Bufferable):
                source.append(s)
            else:
                source.extend(s)
        RestructureFilterbank.__init__(self, tuple(source), type='serial')
        
class Interleave(RestructureFilterbank):
    '''
    Filterbank that interleaves the channels of its inputs, e.g. with two
    input sources with channels AB and CD respectively, the output would have
    channels ACBD. You can initialise with multiple sources separated by
    commas, or by passing a list of sources.
    '''
    def __init__(self, *sources):
        source = []
        for s in sources:
            if isinstance(s, Bufferable):
                source.append(s)
            else:
                source.extend(s)
        RestructureFilterbank.__init__(self, tuple(source), type='interleave')

class FunctionFilterbank(Filterbank):
    '''
    Filterbank that just applies a given function. The function should take
    as many arguments as there are sources.
    
    For example, to half-wave rectify inputs::
    
        FunctionFilterbank(source, lambda x: clip(x, 0, Inf))
        
    The syntax ``lambda x: clip(x, 0, Inf)`` defines a function object that
    takes a single argument ``x`` and returns ``clip(x, 0, Inf)``. The numpy
    function ``clip(x, low, high)`` returns the values of ``x`` clipped between
    ``low`` and ``high`` (so if ``x<low`` it returns ``low``, if ``x>high`` it
    returns ``high``, otherwise it returns ``x``). The symbol ``Inf`` means
    infinity, i.e. no clipping of positive values.
    
    **Technical details**
    
    Note that functions should operate on arrays, in particular on 2D buffered
    segments, which are arrays of shape ``(bufsize, nchannels)``. Typically,
    most standard functions from numpy will work element-wise.
    
    If you want a filterbank that changes the shape of the input (e.g. changes
    the number of channels), set the ``nchannels`` keyword argument to the
    number of output channels.
    '''
    def __init__(self, source, func, nchannels=None,**params):
        if isinstance(source, Bufferable):
            source = (source,)
        Filterbank.__init__(self, source)
        self.func = func
        if nchannels is not None:
            self.nchannels = nchannels
        self.params = params

    def buffer_fetch_next(self, samples):
        start = self.cached_buffer_end
        end = start+samples
        inputs = tuple(s.buffer_fetch(start, end) for s in self.source)
#        print inputs,self.params
        return self.func(*inputs,**self.params)


class SumFilterbank(FunctionFilterbank):
    '''  
    Sum filterbanks together with given weight vectors.
    
    For example, to take the sum of two filterbanks::
    
        SumFilterbank((fb1, fb2))
        
    To take the difference::
    
        SumFilterbank((fb1, fb2), (1, -1))
    '''
    def __init__(self, source, weights=None):
        if weights is None:
            weights = ones(len(source))
        self.weights = weights
        func = lambda *inputs: sum(input*w for input, w in zip(inputs, weights))
        FunctionFilterbank.__init__(self, source, func)


class DoNothingFilterbank(Filterbank):
    '''
    Filterbank that does nothing to its input.
    
    Useful for removing a set of filters without having to rewrite your code.
    Can also be used for simply writing compound derived classes. For example,
    if you want a compound Filterbank that does AFilterbank and then
    BFilterbank, but you want to encapsulate that into a single class, you
    could do::
    
        class ABFilterbank(DoNothingFilterbank):
            def __init__(self, source):
                a = AFilterbank(source)
                b = BFilterbank(a)
                DoNothingFilterbank.__init__(self, b)
                
    However, a more general way of writing compound filterbanks is to use
    :class:`CombinedFilterbank`.
    '''
    def buffer_apply(self, input):
        return input


class ControlFilterbank(Filterbank):
    '''
    Filterbank that can be used for controlling behaviour at runtime
    
    Typically, this class is used to implement a control path in an auditory
    model, modifying some filterbank parameters based on the output of other
    filterbanks (or the same ones).
    
    The controller has a set of input filterbanks whose output values are used
    to modify a set of output filterbanks. The update is done by a user specified
    function or class which is passed these output values. The controller should
    be inserted as the last bank in a chain.
    
    Initialisation arguments:
    
    ``source``
        The source filterbank, the values from this are used unmodified as the
        output of this filterbank.
    ``inputs``
        Either a single filterbank, or sequence of filterbanks which are used
        as inputs to the ``updater``.
    ``targets``
        The filterbank or sequence of filterbanks that are modified by the
        updater.
    ``updater``
        The function or class which does the updating, see below.
    ``max_interval``
        If specified, ensures that the updater is called at least as often
        as this interval (but it may be called more often). Can be specified
        as a time or a number of samples.
    
    **The updater**
    
    The ``updater`` argument can be either a function or class instance. If it
    is a function, it should have a form like::
    
        # A single input
        def updater(input):
            ...
        
        # Two inputs
        def updater(input1, input2):
            ...
        
        # Arbitrary number of inputs
        def updater(*inputs):
            ...
            
    Each argument ``input`` to the function is a numpy array of shape
    ``(numsamples, numchannels)`` where ``numsamples`` is the number of samples
    just computed, and ``numchannels`` is the number of channels in the
    corresponding filterbank. The function is not restricted in what it can
    do with these inputs.
    
    Functions can be used to implement relatively simple controllers, but for
    more complicated situations you may want to maintain some state variables
    for example, and in this case you can use a class. The object ``updater``
    should be an instance of a class that defines the ``__call__`` method
    (with the same syntax as above for functions). In addition, you can
    define a reinitialisation method ``reinit()`` which will be called when
    the ``buffer_init()`` method is called on the filterbank, although this is
    entirely optional.
    
    **Example**
    
    The following will do a simple form of gain control, where the gain
    parameter will drift exponentially towards target_rms/rms with a given time
    constant::
    
        # This class implements the gain (see Filterbank for details)
        class GainFilterbank(Filterbank):
            def __init__(self, source, gain=1.0):
                Filterbank.__init__(self, source)
                self.gain = gain
            def buffer_apply(self, input):
                return self.gain*input
        
        # This is the class for the updater object
        class GainController(object):
            def __init__(self, target, target_rms, time_constant):
                self.target = target
                self.target_rms = target_rms
                self.time_constant = time_constant
            def reinit(self):
                self.sumsquare = 0
                self.numsamples = 0
            def __call__(self, input):
                T = input.shape[0]/self.target.samplerate
                self.sumsquare += sum(input**2)
                self.numsamples += input.size
                rms = sqrt(self.sumsquare/self.numsamples)
                g = self.target.gain
                g_tgt = self.target_rms/rms
                tau = self.time_constant
                self.target.gain = g_tgt+exp(-T/tau)*(g-g_tgt)
    
    And an example of using this with an input ``source``, a target RMS of 0.2
    and a time constant of 50 ms, updating every 10 ms::
    
        gain_fb = GainFilterbank(source)
        updater = GainController(gain_fb, 0.2, 50*ms)
        control = ControlFilterbank(gain_fb, source, gain_fb, updater, 10*ms)            
    '''
    def __init__(self, source, inputs, targets, updater, max_interval=None):
        Filterbank.__init__(self, source)
        if not isinstance(inputs, (list, tuple)):
            inputs = [inputs]
        if not isinstance(targets, (list, tuple)):
            targets = [targets]
        self.inputs = inputs
        self.updater = updater
        if max_interval is not None:
            if not isinstance(max_interval, int):
                max_interval = int(max_interval*source.samplerate)
            for x in inputs+targets:
                x.maximum_buffer_size = max_interval
            self.maximum_buffer_size = max_interval
            
    def buffer_init(self):
        Filterbank.buffer_init(self)
        if hasattr(self.updater, 'reinit'):
            self.updater.reinit()
    
    def buffer_fetch_next(self, samples):
        start = self.next_sample
        self.next_sample += samples
        end = start+samples
        source_input = self.source.buffer_fetch(start, end)
        input_buffers = [x.buffer_fetch(start, end) for x in self.inputs]
        self.updater(*input_buffers)
        return source_input


class CombinedFilterbank(Filterbank):
    '''
    Filterbank that encapsulates a chain of filterbanks internally.
    
    This class should mostly be used by people writing extensions to Brian hears
    rather than by users directly. The purpose is to take an existing chain of
    filterbanks and wrap them up so they appear to the user as a single
    filterbank which can be used exactly as any other filterbank.
    
    In order to do this, derive from this class and in your initialisation
    follow this pattern::
    
        class RectifiedGammatone(CombinedFilterbank):
            def __init__(self, source, cf):
                CombinedFilterbank.__init__(self, source)
                source = self.get_modified_source()
                # At this point, insert your chain of filterbanks acting on
                # the modified source object
                gfb = Gammatone(source, cf)
                rectified = FunctionFilterbank(gfb,
                                lambda input: clip(input, 0, Inf))
                # Finally, set the output filterbank to be the last in your chain
                self.set_output(fb)
    
    This combination of a :class:`Gammatone` and a rectification via a
    :class:`FunctionFilterbank` can now be used as a single filterbank, for
    example::
    
        x = whitenoise(100*ms)
        fb = RectifiedGammatone(x, [1*kHz, 1.5*kHz])
        y = fb.process()

    **Details**
    
    The reason for the ``get_modified_source()`` call is that the source
    attribute of a filterbank can be changed after creation. The modified source
    provides a buffer (in fact, a :class:`DoNothingFilterbank`) so that the
    input to the chain of filters defined by the derived class doesn't need to
    be changed.
    '''
    def __init__(self, source):
        Filterbank.__init__(self, source)

    def get_duration(self):
        if hasattr(self, '_duration'):
            return self._duration
        else:
            return max(Filterbank.get_duration(self), self.output.duration)        

    source = property(fget=lambda self:self._source,
                      fset=lambda self, source:self.change_source(source))
    
    def change_source(self, source):
        Filterbank.change_source(self, source)
        if hasattr(self, '_modified_source'):
            self._modified_source.source = source

    def get_modified_source(self):
        self._modified_source = DoNothingFilterbank(self.source)
        return self._modified_source
    
    def set_output(self, output):
        self.output = output
        self.nchannels = output.nchannels
                    
    def buffer_init(self):
        Filterbank.buffer_init(self)
        self.output.buffer_init()
            
    def buffer_fetch(self, start, end):
        return self.output.buffer_fetch(start, end)



from datamanager import DataManager
import multiprocessing
from Queue import Empty as QueueEmpty
import Tkinter
from brian.utils.progressreporting import make_text_report
import inspect
import time
import os
from numpy import ndarray, zeros

__all__ = ['run_tasks']

# This is the default task class used if the user provides only a function
class FunctionTask(object):
    def __init__(self, func):
        self.func = func
    def __call__(self, *args, **kwds):
        # If the function has a 'report' argument, we pass it the reporter
        # function that will have been passed in kwds (see task_compute)
        fc = self.func.func_code
        if 'report' in fc.co_varnames[:fc.co_argcount] or fc.co_flags&8:
            return self.func(*args, **kwds)
        else:
            return self.func(*args)

def run_tasks(dataman, task, items, gui=True, poolsize=0,
              initargs=None, initkwds=None, verbose=None,
              numitems=None):
    '''
    Run a series of tasks using multiple CPUs on a single computer.
    
    Initialised with arguments:
    
    ``dataman``
        The :class:`~brian.tools.datamanager.DataManager` object used to store
        the results in, see below.
    ``task``
        The task function or class (see below).
    ``items``
        A sequence (e.g. list or iterator) of arguments to be passed to the
        task.
    ``gui=True``
        Whether or not to use a Tkinter based GUI to show progress and terminate
        the task run.
    ``poolsize=0``
        The number of CPUs to use. If the value is 0, use all available CPUs,
        if it is -1 use all but one CPU, etc.
    ``initargs``, ``initkwds``
        If ``task`` is a class, these are the initialisation arguments and
        keywords for the class.
    ``verbose=None``
        Specify True or False to print out every progress message (defaults to
        False if the GUI is used, or True if not).
    ``numitems=None``
        For iterables (rather than fixed length sequences), if you specify the
        number of items, an estimate of the time remaining will be given.
        
    The task (defined by a function or class, see below) will be called on each
    item in ``items``, and the results saved to ``dataman``. Results are stored
    in the format ``(key, val)`` where ``key`` is a unique but meaningless
    identifier. Results can be retrieved using ``dataman.values()`` or (for
    large data sets that should be iterated over) ``dataman.itervalues()``.
    
    The task can either be a function or a class. If it is a function, it will
    be called for each item in ``items``. If the items are tuples, the function
    will be called with those tuples as arguments (e.g. if the item is
    ``(1,2,3)`` the function will be called as ``task(1, 2, 3)``). If the task
    is a class, it can have an ``__init__`` method that is called once for
    each process (each CPU) at the beginning of the task run. If the ``__init__``
    method has a ``process_number`` argument, it will be passed an integer value
    from 0 to ``numprocesses-1`` giving the number of the process (note, this is
    not the process ID). The class should
    define a ``__call__`` method that behaves the same as above for ``task``
    being a function. In both cases (function or class), if the arguments
    include a keyword ``report`` then it will be passed a value that can be
    passed as the ``report`` keyword in Brian's :func:`run` function to give
    feedback on the simulation as it runs. A ``task`` function can also set
    ``self.taskname`` as a string that will be displayed on the GUI to give
    additional information.
    
    .. warning::
        On Windows, make sure that :func:`run_tasks` is only called from
        within a block such as::
        
            if __name__=='__main__':
                run_tasks(...)
                
        Otherwise, the program will go into a recursive loop.
        
    Note that this class only allows you to run tasks on a single computer, to
    distribute work over multiple computers, we suggest using
    `Playdoh <http://code.google.com/p/playdoh/>`__.
    '''
    # User can provide task as a class or a function, if its a function we
    # we use the default FunctionTask
    if not inspect.isclass(task):
        f = task
        initargs = (task,)
        task = FunctionTask
    else:
        f = task.__call__
    fc = f.func_code
    if 'report' in fc.co_varnames[:fc.co_argcount] or fc.co_flags&8:
        will_report = True
    else:
        will_report = False
    if numitems is None and isinstance(items, (list, tuple, ndarray)):
        numitems = len(items)
    # This will be used to provide process safe access to the data manager
    # (so that multiple processes do not attempt to write to the session at
    # the same time)
    session = dataman.locking_computer_session()
    if poolsize<=0:
        numprocesses = poolsize+multiprocessing.cpu_count()
    elif poolsize>0:
        numprocesses = poolsize
    manager = multiprocessing.Manager()
    # We have to send the process number to the initializer via this silly
    # queue because of a limitation of multiprocessing
    process_number_queue = manager.Queue()
    for n in range(numprocesses):
        process_number_queue.put(n)
    # This will be used to send messages about the status of the run, i.e.
    # percentage complete
    message_queue = manager.Queue()
    if initargs is None:
        initargs = ()
    if initkwds is None:
        initkwds = {}
    pool = multiprocessing.Pool(processes=numprocesses,
                                initializer=pool_initializer,
                                initargs=(process_number_queue, message_queue,
                                          dataman, session,
                                          task, initargs, initkwds))
    results = pool.imap_unordered(task_compute, items)
    # We use this to map process IDs to task number, so that we can show the
    # information on the GUI in a consistent fashion
    pid_to_id = dict((pid, i) for i, pid in enumerate([p.pid for p in pool._pool]))
    start = time.time()
    stoprunningsim = [False]
    def terminate_sim():
        # We acquire the datamanager session lock so that if a process is in the
        # middle of writing data, it won't be terminated until its finished,
        # meaning we can safely terminate the process without worrying about
        # data loss.
        session.acquire()
        pool.terminate()
        session.release()
        stoprunningsim[0] = True
    if gui:
        if verbose is None:
            verbose = False
        controller = GuiTaskController(numprocesses, terminate_sim,
                                       verbose=verbose, will_report=will_report)
    else:
        if verbose is None:
            verbose = True
        controller = TextTaskController(numprocesses, terminate_sim, verbose=verbose)
    for i in range(numprocesses):
        controller.update_process(i, 0, 0, 'No task information')
    i = 0
    controller.update_overall(0, numitems)
    def empty_message_queue():
        while not message_queue.empty():
            try:
                pid, taskname, elapsed, complete = message_queue.get_nowait()
                controller.update_process(pid_to_id[pid], elapsed, complete, taskname)
            except QueueEmpty:
                break
        controller.update()
        
    while True:
        try:
            # This waits 0.1s for a new result, and otherwise raises a
            # TimeoutError that allows the GUI to update the percentage
            # complete
            nextresult = results.next(0.1)
            empty_message_queue()
            i = i+1
            elapsed = time.time()-start
            complete = 0.0
            controller.update_overall(i, numitems)
        except StopIteration:
            terminate_sim()
            print 'Finished.'
            break
        except (KeyboardInterrupt, SystemExit):
            terminate_sim()
            print 'Terminated task processes'
            raise
        except multiprocessing.TimeoutError:
            empty_message_queue()
            if stoprunningsim[0]:
                print 'Terminated task processes'
                break
    controller.destroy()


# We store these values as global values, which are initialised by
# pool_initializer on each process
task_object = None
task_dataman = None
task_session = None
task_message_queue = None

def pool_initializer(process_number_queue, message_queue, dataman, session,
                     task, initargs, initkwds):
    global task_object, task_dataman, task_session, task_message_queue
    n = process_number_queue.get()
    init_method = task.__init__
    fc = init_method.func_code
    # Checks if there is a process_number argument explicitly given in the
    # __init__ method of the task class, the co_flags&8 checks i there is a
    # **kwds parameter in the definition
    if 'process_number' in fc.co_varnames[:fc.co_argcount] or fc.co_flags&8:
        initkwds['process_number'] = n
    task_object = task(*initargs, **initkwds)
    task_dataman = dataman
    task_session = session
    task_message_queue = message_queue

def task_reporter(elapsed, complete):
    # If the task class defines a task name, we can display it with the
    # percentage complete
    if hasattr(task_object, 'taskname'):
        taskname = task_object.taskname
    else:
        taskname = 'No task information'
    # This queue is used by the main loop in run_tasks
    task_message_queue.put((os.getpid(), taskname, elapsed, complete))

def task_compute(args):
    if not isinstance(args, tuple):
        args = (args,)
    # We check if the task function has a report argument, and if it does we
    # pass it task_reporter so that it can integrate with the GUI
    kwds = {}
    fc = task_object.__call__.func_code
    if 'report' in fc.co_varnames[:fc.co_argcount] or fc.co_flags&8:
        kwds['report'] = task_reporter
    result = task_object(*args, **kwds)
    # Save the results, with a unique key, to the locking session of the dataman
    task_session[task_dataman.make_unique_key()] = result

class TaskController(object):
    def __init__(self, processes, terminator, verbose=True):
        self.verbose = verbose
        self.completion = zeros(processes)
        self.numitems, self.numdone = None, 0
        self.start_time = time.time()
    def update_process(self, i, elapsed, complete, msg):
        self.completion[i] = complete%1.0
        if self.verbose:
            print 'Process '+str(i)+': '+make_text_report(elapsed, complete)+': '+msg
            _, msg = self.get_overall_completion()
            print msg
    def get_overall_completion(self):
        complete = 0.0
        numitems, numdone = self.numitems, self.numdone
        elapsed = time.time()-self.start_time
        if numitems is not None:
            complete = (numdone+sum(self.completion))/numitems
        txt = 'Overall, '+str(numdone)+' done'
        if numitems is not None:
            txt += ' of '+str(numitems)+': '+make_text_report(elapsed, complete)
        return complete, txt
    def update_overall(self, numdone, numitems):
        self.numdone = numdone
        self.numitems = numitems
    def recompute_overall(self):
        pass
    def update(self):
        pass
    def destroy(self):
        pass

class TextTaskController(TaskController):
    def update_overall(self, numdone, numitems):
        TaskController.update_overall(self, numdone, numitems)
        _, msg = self.get_overall_completion()
        print msg

# task control GUI
class GuiTaskController(Tkinter.Tk, TaskController):
    def __init__(self, processes, terminator, width=600, verbose=False,
                 will_report=True):
        Tkinter.Tk.__init__(self, None)
        TaskController.__init__(self, processes, terminator, verbose=verbose)
        self.parent = None
        self.grid()
        button = Tkinter.Button(self, text='Terminate task',
                                command=terminator)
        button.grid(column=0, row=0)
        self.pb_width = width
        self.progressbars = []
        numbars = 1
        self.will_report = will_report
        if will_report:
            numbars += processes
        for i in xrange(numbars):
            can = Tkinter.Canvas(self, width=width, height=30)
            can.grid(column=0, row=1+i)
            can.create_rectangle(0, 0, width, 30, fill='#aaaaaa')
            if i<numbars-1:
                col = '#ffaaaa'
            else:
                col = '#aaaaff'
            r = can.create_rectangle(0, 0, 0, 30, fill=col, width=0)
            t = can.create_text(width/2, 15, text='')
            self.progressbars.append((can, r, t))
        self.title('Task control')
        
    def update_process(self, i, elapsed, complete, msg):
        TaskController.update_process(self, i, elapsed, complete, msg)
        if self.will_report:
            can, r, t = self.progressbars[i]
            can.itemconfigure(t, text='Process '+str(i)+': '+make_text_report(elapsed, complete)+': '+msg)
            can.coords(r, 0, 0, int(self.pb_width*complete), 30)
        self.recompute_overall()
        
    def update_overall(self, numdone, numitems):
        TaskController.update_overall(self, numdone, numitems)
        self.recompute_overall()
    
    def recompute_overall(self):
        complete, msg = TaskController.get_overall_completion(self)
        numitems = self.numitems
        can, r, t = self.progressbars[-1]
        can.itemconfigure(t, text=msg)
        if numitems is not None:
            can.coords(r, 0, 0, int(self.pb_width*complete), 30)
        self.update()
